/* DreemGL is a collaboration between Teeming Society & Samsung Electronics, sponsored by Samsung and others.
   Copyright 2015-2016 Teeming Society. Licensed under the Apache License, Version 2.0 (the "License"); You may not use this file except in compliance with the License.
   You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing,
   software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and limitations under the License.*/

// Utility to used to generate the markdown file listing all colors in the system, and a
// list of all glyphs in Font Awesome. The appendix can be found at:
// http://localhost:2000/docs/api/index.html#!/guide/appendix

var fs = require('fs');

var color_wikipedia = {
	acidgreen:"0xffB0BF1A",aero:"0xff7CB9E8",aeroblue:"0xffC9FFE5",africanviolet:"0xffB284BE",airforceblueraf:"0xff5D8AA8",airforceblueusaf:"0xff00308F",
	airsuperiorityblue:"0xff72A0C1",alabamacrimson:"0xffAF002A",aliceblue:"0xffF0F8FF",alizarincrimson:"0xffE32636",alloyorange:"0xffC46210",
	almond:"0xffEFDECD",amaranth:"0xffE52B50",amaranthpink:"0xffF19CBB",amaranthpurple:"0xffAB274F",amaranthred:"0xffD3212D",amazon:"0xff3B7A57",
	amber:"0xffFFBF00",ambersaeece:"0xffFF7E00",americanrose:"0xffFF033E",amethyst:"0xff9966CC",androidgreen:"0xffA4C639",antiflashwhite:"0xffF2F3F4",
	antiquebrass:"0xffCD9575",antiquebronze:"0xff665D1E",antiquefuchsia:"0xff915C83",antiqueruby:"0xff841B2D",antiquewhite:"0xffFAEBD7",aoenglish:"0xff008000",
	applegreen:"0xff8DB600",apricot:"0xffFBCEB1",aqua:"0xff00FFFF",aquamarine:"0xff7FFFD4",armygreen:"0xff4B5320",artichoke:"0xff8F9779",arylideyellow:"0xffE9D66B",
	ashgrey:"0xffB2BEB5",asparagus:"0xff87A96B",atomictangerine:"0xffFF9966",auburn:"0xffA52A2A",aureolin:"0xffFDEE00",aurometalsaurus:"0xff6E7F80",
	avocado:"0xff568203",azure:"0xff007FFF",azurecolor:"0xffF0FFFF",azuremist:"0xffF0FFFF",azureishwhite:"0xffDBE9F4",babyblue:"0xff89CFF0",babyblueeyes:"0xffA1CAF1",
	babypink:"0xffF4C2C2",babypowder:"0xffFEFEFA",bakermillerpink:"0xffFF91AF",ballblue:"0xff21ABCD",bananamania:"0xffFAE7B5",bananayellow:"0xffFFE135",
	bangladeshgreen:"0xff006A4E",barbiepink:"0xffE0218A",barnred:"0xff7C0A02",battleshipgrey:"0xff848482",bazaar:"0xff98777B",beaublue:"0xffBCD4E6",
	beaver:"0xff9F8170",beige:"0xffF5F5DC",bdazzledblue:"0xff2E5894",bigdiporuby:"0xff9C2542",bisque:"0xffFFE4C4",bistre:"0xff3D2B1F",bistrebrown:"0xff967117",
	bitterlemon:"0xffCAE00D",bitterlime:"0xffBFFF00",bittersweet:"0xffFE6F5E",bittersweetshimmer:"0xffBF4F51",black:"0xff000000",blackbean:"0xff3D0C02",
	blackleatherjacket:"0xff253529",blackolive:"0xff3B3C36",blanchedalmond:"0xffFFEBCD",blastoffbronze:"0xffA57164",bleudefrance:"0xff318CE7",
	blizzardblue:"0xffACE5EE",blond:"0xffFAF0BE",blue:"0xff0000FF",bluecrayola:"0xff1F75FE",bluemunsell:"0xff0093AF",bluencs:"0xff0087BD",bluepantone:"0xff0018A8",
	bluepigment:"0xff333399",blueryb:"0xff0247FE",bluebell:"0xffA2A2D0",bluegray:"0xff6699CC",bluegreen:"0xff0D98BA",bluemagentaviolet:"0xff553592",
	bluesapphire:"0xff126180",blueviolet:"0xff8A2BE2",blueyonder:"0xff5072A7",blueberry:"0xff4F86F7",bluebonnet:"0xff1C1CF0",blush:"0xffDE5D83",
	bole:"0xff79443B",bondiblue:"0xff0095B6",bone:"0xffE3DAC9",bostonuniversityred:"0xffCC0000",bottlegreen:"0xff006A4E",boysenberry:"0xff873260",
	brandeisblue:"0xff0070FF",brass:"0xffB5A642",brickred:"0xffCB4154",brightcerulean:"0xff1DACD6",brightgreen:"0xff66FF00",brightlavender:"0xffBF94E4",
	brightlilac:"0xffD891EF",brightmaroon:"0xffC32148",brightnavyblue:"0xff1974D2",brightpink:"0xffFF007F",brightturquoise:"0xff08E8DE",brightube:"0xffD19FE8",
	brilliantazure:"0xff3399FF",brilliantlavender:"0xffF4BBFF",brilliantrose:"0xffFF55A3",brinkpink:"0xffFB607F",britishracinggreen:"0xff004225",
	bronze:"0xffCD7F32",bronzeyellow:"0xff737000",browntraditional:"0xff964B00",brown:"0xffA52A2A",brownnose:"0xff6B4423",brunswickgreen:"0xff1B4D3E",
	bubblegum:"0xffFFC1CC",bubbles:"0xffE7FEFF",buff:"0xffF0DC82",budgreen:"0xff7BB661",bulgarianrose:"0xff480607",burgundy:"0xff800020",burlywood:"0xffDEB887",
	burntorange:"0xffCC5500",burntsienna:"0xffE97451",burntumber:"0xff8A3324",byzantine:"0xffBD33A4",byzantium:"0xff702963",cadet:"0xff536872",
	cadetblue:"0xff5F9EA0",cadetgrey:"0xff91A3B0",cadmiumgreen:"0xff006B3C",cadmiumorange:"0xffED872D",cadmiumred:"0xffE30022",cadmiumyellow:"0xffFFF600",
	cafeaulait:"0xffA67B5B",cafenoir:"0xff4B3621",calpolypomonagreen:"0xff1E4D2B",cambridgeblue:"0xffA3C1AD",camel:"0xffC19A6B",cameopink:"0xffEFBBCC",
	camouflagegreen:"0xff78866B",canaryyellow:"0xffFFEF00",candyapplered:"0xffFF0800",candypink:"0xffE4717A",capri:"0xff00BFFF",caputmortuum:"0xff592720",
	cardinal:"0xffC41E3A",caribbeangreen:"0xff00CC99",carmine:"0xff960018",carminemp:"0xffD70040",carminepink:"0xffEB4C42",carminered:"0xffFF0038",
	carnationpink:"0xffFFA6C9",carnelian:"0xffB31B1B",carolinablue:"0xff56A0D3",carrotorange:"0xffED9121",castletongreen:"0xff00563F",catalinablue:"0xff062A78",
	catawba:"0xff703642",cedarchest:"0xffC95A49",ceil:"0xff92A1CF",celadon:"0xffACE1AF",celadonblue:"0xff007BA7",celadongreen:"0xff2F847C",celeste:"0xffB2FFFF",
	celestialblue:"0xff4997D0",cerise:"0xffDE3163",cerisepink:"0xffEC3B83",cerulean:"0xff007BA7",ceruleanblue:"0xff2A52BE",ceruleanfrost:"0xff6D9BC3",
	cgblue:"0xff007AA5",cgred:"0xffE03C31",chamoisee:"0xffA0785A",champagne:"0xffF7E7CE",charcoal:"0xff36454F",charlestongreen:"0xff232B2B",charmpink:"0xffE68FAC",
	chartreusetraditional:"0xffDFFF00",chartreuse:"0xff7FFF00",cherry:"0xffDE3163",cherryblossompink:"0xffFFB7C5",chestnut:"0xff954535",chinapink:"0xffDE6FA1",
	chinarose:"0xffA8516E",chinesered:"0xffAA381E",chineseviolet:"0xff856088",chocolatetraditional:"0xff7B3F00",chocolate:"0xffD2691E",chromeyellow:"0xffFFA700",
	cinereous:"0xff98817B",cinnabar:"0xffE34234",cinnamon:"0xffD2691E",citrine:"0xffE4D00A",citron:"0xff9FA91F",claret:"0xff7F1734",classicrose:"0xffFBCCE7",
	cobalt:"0xff0047AB",cocoabrown:"0xffD2691E",coconut:"0xff965A3E",coffee:"0xff6F4E37",columbiablue:"0xffC4D8E2",congopink:"0xffF88379",coolblack:"0xff000000",
	coolgrey:"0xff8C92AC",copper:"0xffB87333",coppercrayola:"0xffDA8A67",copperpenny:"0xffAD6F69",copperred:"0xffCB6D51",copperrose:"0xff996666",
	coquelicot:"0xffFF3800",coral:"0xffFF7F50",coralpink:"0xffF88379",coralred:"0xffFF4040",cordovan:"0xff893F45",corn:"0xffFBEC5D",cornellred:"0xffB31B1B",
	cornflowerblue:"0xff6495ED",cornsilk:"0xffFFF8DC",cosmiclatte:"0xffFFF8E7",coyotebrown:"0xff81613e",cottoncandy:"0xffFFBCD9",cream:"0xffFFFDD0",
	crimson:"0xffDC143C",crimsonglory:"0xffBE0032",crimsonred:"0xff990000",cyan:"0xff00FFFF",cyanazure:"0xff4E82b4",cyancobaltblue:"0xff28589C",
	cyancornflowerblue:"0xff188BC2",cyanprocess:"0xff00B7EB",cybergrape:"0xff58427C",cyberyellow:"0xffFFD300",daffodil:"0xffFFFF31",dandelion:"0xffF0E130",
	darkblue:"0xff00008B",darkbluegray:"0xff666699",darkbrown:"0xff654321",darkbyzantium:"0xff5D3954",darkcandyapplered:"0xffA40000",darkcerulean:"0xff08457E",
	darkchestnut:"0xff986960",darkcoral:"0xffCD5B45",darkcyan:"0xff008B8B",darkelectricblue:"0xff536878",darkgoldenrod:"0xffB8860B",darkgrayx11:"0xffA9A9A9",
	darkgreen:"0xff013220",darkgreenx11:"0xff006400",darkimperialblue:"0xff00416A",darkjunglegreen:"0xff1A2421",darkkhaki:"0xffBDB76B",darklava:"0xff483C32",
	darklavender:"0xff734F96",darkliver:"0xff534B4F",darkliverhorses:"0xff543D37",darkmagenta:"0xff8B008B",darkmediumgray:"0xffA9A9A9",darkmidnightblue:"0xff003366",
	darkmossgreen:"0xff4A5D23",darkolivegreen:"0xff556B2F",darkorange:"0xffFF8C00",darkorchid:"0xff9932CC",darkpastelblue:"0xff779ECB",darkpastelgreen:"0xff03C03C",
	darkpastelpurple:"0xff966FD6",darkpastelred:"0xffC23B22",darkpink:"0xffE75480",darkpowderblue:"0xff003399",darkpuce:"0xff4F3A3C",darkraspberry:"0xff872657",
	darkred:"0xff8B0000",darksalmon:"0xffE9967A",darkscarlet:"0xff560319",darkseagreen:"0xff8FBC8F",darksienna:"0xff3C1414",darkskyblue:"0xff8CBED6",
	darkslateblue:"0xff483D8B",darkslategray:"0xff2F4F4F",darkspringgreen:"0xff177245",darktan:"0xff918151",darktangerine:"0xffFFA812",darktaupe:"0xff483C32",
	darkterracotta:"0xffCC4E5C",darkturquoise:"0xff00CED1",darkvanilla:"0xffD1BEA8",darkviolet:"0xff9400D3",darkyellow:"0xff9B870C",dartmouthgreen:"0xff00703C",
	davysgrey:"0xff555555",debianred:"0xffD70A53",deepcarmine:"0xffA9203E",deepcarminepink:"0xffEF3038",deepcarrotorange:"0xffE9692C",deepcerise:"0xffDA3287",
	deepchampagne:"0xffFAD6A5",deepchestnut:"0xffB94E48",deepcoffee:"0xff704241",deepfuchsia:"0xffC154C1",deepgreen:"0xff056608",deepgreencyanturquoise:"0xff0E7C61",
	deepjunglegreen:"0xff004B49",deeplemon:"0xffF5C71A",deeplilac:"0xff9955BB",deepmagenta:"0xffCC00CC",deepmauve:"0xffD473D4",deepmossgreen:"0xff355E3B",
	deeppeach:"0xffFFCBA4",deeppink:"0xffFF1493",deeppuce:"0xffA95C68",deepruby:"0xff843F5B",deepsaffron:"0xffFF9933",deepskyblue:"0xff00BFFF",
	deepspacesparkle:"0xff4A646C",deepspringbud:"0xff556B2F",deeptaupe:"0xff7E5E60",deeptuscanred:"0xff66424D",deer:"0xffBA8759",denim:"0xff1560BD",
	desaturatedcyan:"0xff669999",desert:"0xffC19A6B",desertsand:"0xffEDC9AF",desire:"0xffEA3C53",diamond:"0xffB9F2FF",dimgray:"0xff696969",dirt:"0xff9B7653",
	dodgerblue:"0xff1E90FF",dogwoodrose:"0xffD71868",dollarbill:"0xff85BB65",donkeybrown:"0xff664C28",drab:"0xff967117",dukeblue:"0xff00009C",duststorm:"0xffE5CCC9",
	dutchwhite:"0xffEFDFBB",earthyellow:"0xffE1A95F",ebony:"0xff555D50",ecru:"0xffC2B280",eerieblack:"0xff1B1B1B",eggplant:"0xff614051",eggshell:"0xffF0EAD6",
	egyptianblue:"0xff1034A6",electricblue:"0xff7DF9FF",electriccrimson:"0xffFF003F",electriccyan:"0xff00FFFF",electricgreen:"0xff00FF00",electricindigo:"0xff6F00FF",
	electriclavender:"0xffF4BBFF",electriclime:"0xffCCFF00",electricpurple:"0xffBF00FF",electricultramarine:"0xff3F00FF",electricviolet:"0xff8F00FF",
	electricyellow:"0xffFFFF33",emerald:"0xff50C878",eminence:"0xff6C3082",englishgreen:"0xff1B4D3E",englishlavender:"0xffB48395",englishred:"0xffAB4B52",
	englishviolet:"0xff563C5C",etonblue:"0xff96C8A2",eucalyptus:"0xff44D7A8",fallow:"0xffC19A6B",falured:"0xff801818",fandango:"0xffB53389",fandangopink:"0xffDE5285",
	fashionfuchsia:"0xffF400A1",fawn:"0xffE5AA70",feldgrau:"0xff4D5D53",feldspar:"0xffFDD5B1",ferngreen:"0xff4F7942",ferrarired:"0xffFF2800",fielddrab:"0xff6C541E",
	firebrick:"0xffB22222",fireenginered:"0xffCE2029",flame:"0xffE25822",flamingopink:"0xffFC8EAC",flattery:"0xff6B4423",flavescent:"0xffF7E98E",
	flax:"0xffEEDC82",flirt:"0xffA2006D",floralwhite:"0xffFFFAF0",fluorescentorange:"0xffFFBF00",fluorescentpink:"0xffFF1493",fluorescentyellow:"0xffCCFF00",
	folly:"0xffFF004F",forestgreentraditional:"0xff014421",forestgreen:"0xff228B22",frenchbeige:"0xffA67B5B",frenchbistre:"0xff856D4D",frenchblue:"0xff0072BB",
	frenchfuchsia:"0xffFD3F92",frenchlilac:"0xff86608E",frenchlime:"0xff9EFD38",frenchmauve:"0xffD473D4",frenchpink:"0xffFD6C9E",frenchplum:"0xff811453",
	frenchpuce:"0xff4E1609",frenchraspberry:"0xffC72C48",frenchrose:"0xffF64A8A",frenchskyblue:"0xff77B5FE",frenchviolet:"0xff8806CE",frenchwine:"0xffAC1E44",
	freshair:"0xffA6E7FF",fuchsia:"0xffFF00FF",fuchsiacrayola:"0xffC154C1",fuchsiapink:"0xffFF77FF",fuchsiapurple:"0xffCC397B",fuchsiarose:"0xffC74375",
	fulvous:"0xffE48400",fuzzywuzzy:"0xffCC6666",gainsboro:"0xffDCDCDC",gamboge:"0xffE49B0F",gambogeorangebrown:"0xff996600",genericviridian:"0xff007F66",
	ghostwhite:"0xffF8F8FF",giantsorange:"0xffFE5A1D",ginger:"0xffB06500",glaucous:"0xff6082B6",glitter:"0xffE6E8FA",gogreen:"0xff00AB66",goldmetallic:"0xffD4AF37",
	goldgolden:"0xffFFD700",goldfusion:"0xff85754E",goldenbrown:"0xff996515",goldenpoppy:"0xffFCC200",goldenyellow:"0xffFFDF00",goldenrod:"0xffDAA520",
	grannysmithapple:"0xffA8E4A0",grape:"0xff6F2DA8",gray:"0xff808080",grayx11:"0xffBEBEBE",grayasparagus:"0xff465945",
	grayblue:"0xff8C92AC",greencolorwheelx11green:"0xff00FF00",greencrayola:"0xff1CAC78",green:"0xff008000",greenmunsell:"0xff00A877",
	greenncs:"0xff009F6B",greenpantone:"0xff00AD43",greenpigment:"0xff00A550",greenryb:"0xff66B032",greenyellow:"0xffADFF2F",grizzly:"0xff885818",
	grullo:"0xffA99A86",guppiegreen:"0xff00FF7F",halayaube:"0xff663854",hanblue:"0xff446CCF",hanpurple:"0xff5218FA",hansayellow:"0xffE9D66B",harlequin:"0xff3FFF00",
	harlequingreen:"0xff46CB18",harvardcrimson:"0xffC90016",harvestgold:"0xffDA9100",heartgold:"0xff808000",heliotrope:"0xffDF73FF",heliotropegray:"0xffAA98A9",
	heliotropemagenta:"0xffAA00BB",hollywoodcerise:"0xffF400A1",honeydew:"0xffF0FFF0",honolulublue:"0xff006DB0",hookersgreen:"0xff49796B",hotmagenta:"0xffFF1DCE",
	hotpink:"0xffFF69B4",huntergreen:"0xff355E3B",iceberg:"0xff71A6D2",icterine:"0xffFCF75E",illuminatingemerald:"0xff319177",imperial:"0xff602F6B",
	imperialblue:"0xff002395",imperialpurple:"0xff66023C",imperialred:"0xffED2939",inchworm:"0xffB2EC5D",independence:"0xff4C516D",indiagreen:"0xff138808",
	indianred:"0xffCD5C5C",indianyellow:"0xffE3A857",indigo:"0xff6F00FF",indigodye:"0xff091F92",indigo:"0xff4B0082",internationalkleinblue:"0xff002FA7",
	internationalorangeaerospace:"0xffFF4F00",internationalorangeengineering:"0xffBA160C",internationalorangegoldengatebridge:"0xffC0362C",
	iris:"0xff5A4FCF",irresistible:"0xffB3446C",isabelline:"0xffF4F0EC",islamicgreen:"0xff009000",italianskyblue:"0xffB2FFFF",ivory:"0xffFFFFF0",
	jade:"0xff00A86B",japanesecarmine:"0xff9D2933",japaneseindigo:"0xff264348",japaneseviolet:"0xff5B3256",jasmine:"0xffF8DE7E",jasper:"0xffD73B3E",
	jazzberryjam:"0xffA50B5E",jellybean:"0xffDA614E",jet:"0xff343434",jonquil:"0xffF4CA16",jordyblue:"0xff8AB9F1",junebud:"0xffBDDA57",junglegreen:"0xff29AB87",
	kellygreen:"0xff4CBB17",kenyancopper:"0xff7C1C05",keppel:"0xff3AB09E",khaki:"0xffC3B091",jawad:"0xffC3B091",lightkhaki:"0xffF0E68C",
	kobe:"0xff882D17",kobi:"0xffE79FC4",kombugreen:"0xff354230",kucrimson:"0xffE8000D",lasallegreen:"0xff087830",languidlavender:"0xffD6CADD",lapislazuli:"0xff26619C",
	laserlemon:"0xffFFFF66",laurelgreen:"0xffA9BA9D",lava:"0xffCF1020",lavenderfloral:"0xffB57EDC",lavender:"0xffE6E6FA",lavenderblue:"0xffCCCCFF",
	lavenderblush:"0xffFFF0F5",lavendergray:"0xffC4C3D0",lavenderindigo:"0xff9457EB",lavendermagenta:"0xffEE82EE",lavendermist:"0xffE6E6FA",
	lavenderpink:"0xffFBAED2",lavenderpurple:"0xff967BB6",lavenderrose:"0xffFBA0E3",lawngreen:"0xff7CFC00",lemon:"0xffFFF700",lemonchiffon:"0xffFFFACD",
	lemoncurry:"0xffCCA01D",lemonglacier:"0xffFDFF00",lemonlime:"0xffE3FF00",lemonmeringue:"0xffF6EABE",lemonyellow:"0xffFFF44F",lenurple:"0xffBA93D8",
	licorice:"0xff1A1110",liberty:"0xff545AA7",lightapricot:"0xffFDD5B1",lightblue:"0xffADD8E6",lightbrown:"0xffB5651D",lightcarminepink:"0xffE66771",
	lightcoral:"0xffF08080",lightcornflowerblue:"0xff93CCEA",lightcrimson:"0xffF56991",lightcyan:"0xffE0FFFF",lightdeeppink:"0xffFF5CCD",lightfrenchbeige:"0xffC8AD7F",
	lightfuchsiapink:"0xffF984EF",lightgoldenrodyellow:"0xffFAFAD2",lightgray:"0xffD3D3D3",lightgrayishmagenta:"0xffCC99CC",lightgreen:"0xff90EE90",
	lighthotpink:"0xffFFB3DE",lightkhaki:"0xffF0E68C",lightmediumorchid:"0xffD39BCB",lightmossgreen:"0xffADDFAD",lightorchid:"0xffE6A8D7",lightpastelpurple:"0xffB19CD9",
	lightpink:"0xffFFB6C1",lightredochre:"0xffE97451",lightsalmon:"0xffFFA07A",lightsalmonpink:"0xffFF9999",lightseagreen:"0xff20B2AA",lightskyblue:"0xff87CEFA",
	lightslategray:"0xff778899",lightsteelblue:"0xffB0C4DE",lighttaupe:"0xffB38B6D",lightthulianpink:"0xffE68FAC",lightyellow:"0xffFFFFE0",lilac:"0xffC8A2C8",
	limecolorwheel:"0xffBFFF00",limex11green:"0xff00FF00",limegreen:"0xff32CD32",limerick:"0xff9DC209",lincolngreen:"0xff195905",linen:"0xffFAF0E6",
	lion:"0xffC19A6B",liseranpurple:"0xffDE6FA1",littleboyblue:"0xff6CA0DC",liver:"0xff674C47",liverdogs:"0xffB86D29",liverorgan:"0xff6C2E1F",liverchestnut:"0xff987456",
	livid:"0xff6699CC",lumber:"0xffFFE4CD",lust:"0xffE62020",magenta:"0xffFF00FF",magentacrayola:"0xffFF55A3",magentadye:"0xffCA1F7B",magentapantone:"0xffD0417E",
	magentaprocess:"0xffFF0090",magentahaze:"0xff9F4576",magentapink:"0xffCC338B",magicmint:"0xffAAF0D1",magnolia:"0xffF8F4FF",mahogany:"0xffC04000",
	maize:"0xffFBEC5D",majorelleblue:"0xff6050DC",malachite:"0xff0BDA51",manatee:"0xff979AAA",mangotango:"0xffFF8243",mantis:"0xff74C365",mardigras:"0xff880085",
	marooncrayola:"0xffC32148",maroon:"0xff800000",maroonx11:"0xffB03060",mauve:"0xffE0B0FF",mauvetaupe:"0xff915F6D",mauvelous:"0xffEF98AA",
	maygreen:"0xff4C9141",mayablue:"0xff73C2FB",meatbrown:"0xffE5B73B",mediumaquamarine:"0xff66DDAA",mediumblue:"0xff0000CD",mediumcandyapplered:"0xffE2062C",
	mediumcarmine:"0xffAF4035",mediumchampagne:"0xffF3E5AB",mediumelectricblue:"0xff035096",mediumjunglegreen:"0xff1C352D",mediumlavendermagenta:"0xffDDA0DD",
	mediumorchid:"0xffBA55D3",mediumpersianblue:"0xff0067A5",mediumpurple:"0xff9370DB",mediumredviolet:"0xffBB3385",mediumruby:"0xffAA4069",
	mediumseagreen:"0xff3CB371",mediumskyblue:"0xff80DAEB",mediumslateblue:"0xff7B68EE",mediumspringbud:"0xffC9DC87",mediumspringgreen:"0xff00FA9A",
	mediumtaupe:"0xff674C47",mediumturquoise:"0xff48D1CC",mediumtuscanred:"0xff79443B",mediumvermilion:"0xffD9603B",mediumvioletred:"0xffC71585",
	mellowapricot:"0xffF8B878",mellowyellow:"0xffF8DE7E",melon:"0xffFDBCB4",metallicseaweed:"0xff0A7E8C",metallicsunburst:"0xff9C7C38",mexicanpink:"0xffE4007C",
	midnightblue:"0xff191970",midnightgreeneaglegreen:"0xff004953",mikadoyellow:"0xffFFC40C",mindaro:"0xffE3F988",mint:"0xff3EB489",mintcream:"0xffF5FFFA",
	mintgreen:"0xff98FF98",mistyrose:"0xffFFE4E1",moccasin:"0xffFAEBD7",modebeige:"0xff967117",moonstoneblue:"0xff73A9C2",mordantred19:"0xffAE0C00",
	mossgreen:"0xff8A9A5B",mountainmeadow:"0xff30BA8F",mountbattenpink:"0xff997A8D",msugreen:"0xff18453B",mughalgreen:"0xff306030",mulberry:"0xffC54B8C",
	mustard:"0xffFFDB58",myrtlegreen:"0xff317873",nadeshikopink:"0xffF6ADC6",napiergreen:"0xff2A8000",naplesyellow:"0xffFADA5E",navajowhite:"0xffFFDEAD",
	navy:"0xff000080",navypurple:"0xff9457EB",neoncarrot:"0xffFFA343",neonfuchsia:"0xffFE4164",neongreen:"0xff39FF14",newcar:"0xff214FC6",newyorkpink:"0xffD7837F",
	nonphotoblue:"0xffA4DDED",northtexasgreen:"0xff059033",nyanza:"0xffE9FFDB",oceanboatblue:"0xff0077BE",ochre:"0xffCC7722",officegreen:"0xff008000",
	oldburgundy:"0xff43302E",oldgold:"0xffCFB53B",oldheliotrope:"0xff563C5C",oldlace:"0xffFDF5E6",oldlavender:"0xff796878",oldmauve:"0xff673147",
	oldmossgreen:"0xff867E36",oldrose:"0xffC08081",oldsilver:"0xff848482",olive:"0xff808000",olivedrab3:"0xff6B8E23",olivedrab7:"0xff3C341F",
	olivine:"0xff9AB973",onyx:"0xff353839",operamauve:"0xffB784A7",orangecolorwheel:"0xffFF7F00",orangecrayola:"0xffFF7538",orangepantone:"0xffFF5800",
	orangeryb:"0xffFB9902",orange:"0xffFFA500",orangepeel:"0xffFF9F00",orangered:"0xffFF4500",orchid:"0xffDA70D6",orchidpink:"0xffF2BDCD",oriolesorange:"0xffFB4F14",
	otterbrown:"0xff654321",outerspace:"0xff414A4C",outrageousorange:"0xffFF6E4A",oxfordblue:"0xff002147",oucrimsonred:"0xff990000",pakistangreen:"0xff006600",
	palatinateblue:"0xff273BE2",palatinatepurple:"0xff682860",paleaqua:"0xffBCD4E6",paleblue:"0xffAFEEEE",palebrown:"0xff987654",palecarmine:"0xffAF4035",
	palecerulean:"0xff9BC4E2",palechestnut:"0xffDDADAF",palecopper:"0xffDA8A67",palecornflowerblue:"0xffABCDEF",palegold:"0xffE6BE8A",palegoldenrod:"0xffEEE8AA",
	palegreen:"0xff98FB98",palelavender:"0xffDCD0FF",palemagenta:"0xffF984E5",palepink:"0xffFADADD",paleplum:"0xffDDA0DD",paleredviolet:"0xffDB7093",
	palerobineggblue:"0xff96DED1",palesilver:"0xffC9C0BB",palespringbud:"0xffECEBBD",paletaupe:"0xffBC987E",paleturquoise:"0xffAFEEEE",paleviolet:"0xffCC99FF",
	palevioletred:"0xffDB7093",pansypurple:"0xff78184A",paoloveronesegreen:"0xff009B7D",papayawhip:"0xffFFEFD5",paradisepink:"0xffE63E62",parisgreen:"0xff50C878",
	pastelblue:"0xffAEC6CF",pastelbrown:"0xff836953",pastelgray:"0xffCFCFC4",pastelgreen:"0xff77DD77",pastelmagenta:"0xffF49AC2",pastelorange:"0xffFFB347",
	pastelpink:"0xffDEA5A4",pastelpurple:"0xffB39EB5",pastelred:"0xffFF6961",pastelviolet:"0xffCB99C9",pastelyellow:"0xffFDFD96",patriarch:"0xff800080",
	paynesgrey:"0xff536878",peach:"0xffFFE5B4",peach:"0xffFFCBA4",peachorange:"0xffFFCC99",peachpuff:"0xffFFDAB9",peachyellow:"0xffFADFAD",pear:"0xffD1E231",
	pearl:"0xffEAE0C8",pearlaqua:"0xff88D8C0",pearlypurple:"0xffB768A2",peridot:"0xffE6E200",periwinkle:"0xffCCCCFF",persianblue:"0xff1C39BB",persiangreen:"0xff00A693",
	persianindigo:"0xff32127A",persianorange:"0xffD99058",persianpink:"0xffF77FBE",persianplum:"0xff701C1C",persianred:"0xffCC3333",persianrose:"0xffFE28A2",
	persimmon:"0xffEC5800",peru:"0xffCD853F",phlox:"0xffDF00FF",phthaloblue:"0xff000F89",phthalogreen:"0xff123524",pictonblue:"0xff45B1E8",pictorialcarmine:"0xffC30B4E",
	piggypink:"0xffFDDDE6",pinegreen:"0xff01796F",pineapple:"0xff563C5C",pink:"0xffFFC0CB",pinkpantone:"0xffD74894",pinklace:"0xffFFDDF4",pinklavender:"0xffD8B2D1",
	pinkpearl:"0xffE7ACCF",pinksherbet:"0xffF78FA7",pistachio:"0xff93C572",platinum:"0xffE5E4E2",plum:"0xff8E4585",plum:"0xffDDA0DD",pompandpower:"0xff86608E",
	popstar:"0xffBE4F62",portlandorange:"0xffFF5A36",powderblue:"0xffB0E0E6",princetonorange:"0xffF58025",prune:"0xff701C1C",prussianblue:"0xff003153",
	psychedelicpurple:"0xffDF00FF",puce:"0xffCC8899",pucered:"0xff722F37",pullmanbrownupsbrown:"0xff644117",pumpkin:"0xffFF7518",purple:"0xff800080",
	purplemunsell:"0xff9F00C5",purplex11:"0xffA020F0",purpleheart:"0xff69359C",purplemountainmajesty:"0xff9678B6",purplenavy:"0xff4E5180",purplepizzazz:"0xffFE4EDA",
	purpletaupe:"0xff50404D",purpureus:"0xff9A4EAE",quartz:"0xff51484F",queenblue:"0xff436B95",queenpink:"0xffE8CCD7",quinacridonemagenta:"0xff8E3A59",
	rackley:"0xff5D8AA8",radicalred:"0xffFF355E",rajah:"0xffFBAB60",raspberry:"0xffE30B5D",raspberryglace:"0xff915F6D",raspberrypink:"0xffE25098",
	raspberryrose:"0xffB3446C",rawumber:"0xff826644",razzledazzlerose:"0xffFF33CC",razzmatazz:"0xffE3256B",razzmicberry:"0xff8D4E85",red:"0xffFF0000",
	redcrayola:"0xffEE204D",redmunsell:"0xffF2003C",redncs:"0xffC40233",redpantone:"0xffED2939",redpigment:"0xffED1C24",redryb:"0xffFE2712",redbrown:"0xffA52A2A",
	reddevil:"0xff860111",redorange:"0xffFF5349",redpurple:"0xffE40078",redviolet:"0xffC71585",redwood:"0xffA45A52",regalia:"0xff522D80",resolutionblue:"0xff002387",
	rhythm:"0xff777696",richblack:"0xff004040",richbrilliantlavender:"0xffF1A7FE",richcarmine:"0xffD70040",richelectricblue:"0xff0892D0",richlavender:"0xffA76BCF",
	richlilac:"0xffB666D2",richmaroon:"0xffB03060",riflegreen:"0xff444C38",roastcoffee:"0xff704241",robineggblue:"0xff00CCCC",rocketmetallic:"0xff8A7F80",
	romansilver:"0xff838996",rose:"0xffFF007F",rosebonbon:"0xffF9429E",roseebony:"0xff674846",rosegold:"0xffB76E79",rosemadder:"0xffE32636",rosepink:"0xffFF66CC",
	rosequartz:"0xffAA98A9",rosered:"0xffC21E56",rosetaupe:"0xff905D5D",rosevale:"0xffAB4E52",rosewood:"0xff65000B",rossocorsa:"0xffD40000",rosybrown:"0xffBC8F8F",
	royalazure:"0xff0038A8",royalblue1:"0xff002366",royalblue2:"0xff4169E1",royalfuchsia:"0xffCA2C92",royalpurple:"0xff7851A9",royalyellow:"0xffFADA5E",
	ruber:"0xffCE4676",rubinered:"0xffD10056",ruby:"0xffE0115F",rubyred:"0xff9B111E",ruddy:"0xffFF0028",ruddybrown:"0xffBB6528",ruddypink:"0xffE18E96",
	rufous:"0xffA81C07",russet:"0xff80461B",russiangreen:"0xff679267",russianviolet:"0xff32174D",rust:"0xffB7410E",rustyred:"0xffDA2C43",sacramentostategreen:"0xff00563F",
	saddlebrown:"0xff8B4513",safetyorangeblazeorange:"0xffFF6700",safetyyellow:"0xffEED202",saffron:"0xffF4C430",sage:"0xffBCB88A",stpatricksblue:"0xff23297A",
	salmon:"0xffFA8072",salmonpink:"0xffFF91A4",sand:"0xffC2B280",sanddune:"0xff967117",sandstorm:"0xffECD540",sandybrown:"0xffF4A460",sandytaupe:"0xff967117",
	sangria:"0xff92000A",sapgreen:"0xff507D2A",sapphire:"0xff0F52BA",sapphireblue:"0xff0067A5",satinsheengold:"0xffCBA135",scarlet:"0xffFF2400",
	scarlet:"0xffFD0E35",schausspink:"0xffFF91AF",schoolbusyellow:"0xffFFD800",screamingreen:"0xff76FF7A",seablue:"0xff006994",seagreen:"0xff2E8B57",
	sealbrown:"0xff321414",seashell:"0xffFFF5EE",selectiveyellow:"0xffFFBA00",sepia:"0xff704214",shadow:"0xff8A795D",shadowblue:"0xff778BA5",shampoo:"0xffFFCFF1",
	shamrockgreen:"0xff009E60",sheengreen:"0xff8FD400",shimmeringblush:"0xffD98695",shockingpink:"0xffFC0FC0",shockingpinkcrayola:"0xffFF6FFF",
	sienna:"0xff882D17",silver:"0xffC0C0C0",silverchalice:"0xffACACAC",silverlakeblue:"0xff5D89BA",silverpink:"0xffC4AEAD",silversand:"0xffBFC1C2",
	sinopia:"0xffCB410B",skobeloff:"0xff007474",skyblue:"0xff87CEEB",skymagenta:"0xffCF71AF",slateblue:"0xff6A5ACD",slategray:"0xff708090",smaltdarkpowderblue:"0xff003399",
	smitten:"0xffC84186",smoke:"0xff738276",smokyblack:"0xff100C08",smokytopaz:"0xff933D41",snow:"0xffFFFAFA",soap:"0xffCEC8EF",solidpink:"0xff893843",
	sonicsilver:"0xff757575",spartancrimson:"0xff9E1316",spacecadet:"0xff1D2951",spanishbistre:"0xff807532",spanishblue:"0xff0070B8",spanishcarmine:"0xffD10047",
	spanishcrimson:"0xffE51A4C",spanishgray:"0xff989898",spanishgreen:"0xff009150",spanishorange:"0xffE86100",spanishpink:"0xffF7BFBE",spanishred:"0xffE60026",
	spanishskyblue:"0xff00FFFF",spanishviolet:"0xff4C2882",spanishviridian:"0xff007F5C",spirodiscoball:"0xff0FC0FC",springbud:"0xffA7FC00",springgreen:"0xff00FF7F",
	starcommandblue:"0xff007BB8",steelblue:"0xff4682B4",steelpink:"0xffCC33CC",stildegrainyellow:"0xffFADA5E",stizza:"0xff990000",stormcloud:"0xff4F666A",
	straw:"0xffE4D96F",strawberry:"0xffFC5A8D",sunglow:"0xffFFCC33",sunray:"0xffE3AB57",sunset:"0xffFAD6A5",sunsetorange:"0xffFD5E53",superpink:"0xffCF6BA9",
	tan:"0xffD2B48C",tangelo:"0xffF94D00",tangerine:"0xffF28500",tangerineyellow:"0xffFFCC00",tangopink:"0xffE4717A",taupe:"0xff483C32",taupegray:"0xff8B8589",
	teagreen:"0xffD0F0C0",tearose:"0xffF88379",tearose:"0xffF4C2C2",teal:"0xff008080",tealblue:"0xff367588",tealdeer:"0xff99E6B3",tealgreen:"0xff00827F",
	telemagenta:"0xffCF3476",tenne:"0xffCD5700",terracotta:"0xffE2725B",thistle:"0xffD8BFD8",thulianpink:"0xffDE6FA1",ticklemepink:"0xffFC89AC",
	tiffanyblue:"0xff0ABAB5",tigerseye:"0xffE08D3C",timberwolf:"0xffDBD7D2",titaniumyellow:"0xffEEE600",tomato:"0xffFF6347",toolbox:"0xff746CC0",
	topaz:"0xffFFC87C",tractorred:"0xffFD0E35",trolleygrey:"0xff808080",tropicalrainforest:"0xff00755E",trueblue:"0xff0073CF",tuftsblue:"0xff417DC1",
	tulip:"0xffFF878D",tumbleweed:"0xffDEAA88",tumblr:"0xff2C4762",turkishrose:"0xffB57281",turquoise:"0xff40E0D0",turquoiseblue:"0xff00FFEF",turquoisegreen:"0xffA0D6B4",
	tuscan:"0xffFAD6A5",tuscanbrown:"0xff6F4E37",tuscanred:"0xff7C4848",tuscantan:"0xffA67B5B",tuscany:"0xffC09999",twilightlavender:"0xff8A496B",
	tyrianpurple:"0xff66023C",uablue:"0xff0033AA",uared:"0xffD9004C",ube:"0xff8878C3",uclablue:"0xff536895",uclagold:"0xffFFB300",ufogreen:"0xff3CD070",
	ultramarine:"0xff120A8F",ultramarineblue:"0xff4166F5",ultrapink:"0xffFF6FFF",ultrared:"0xffFC6C85",umber:"0xff635147",unbleachedsilk:"0xffFFDDCA",
	unitednationsblue:"0xff5B92E5",universityofcaliforniagold:"0xffB78727",unmellowyellow:"0xffFFFF66",upforestgreen:"0xff014421",upmaroon:"0xff7B1113",
	upsdellred:"0xffAE2029",urobilin:"0xffE1AD21",usafablue:"0xff004F98",usccardinal:"0xff990000",uscgold:"0xffFFCC00",universityoftennesseeorange:"0xffF77F00",
	utahcrimson:"0xffD3003F",vanilla:"0xffF3E5AB",vanillaice:"0xffF38FA9",vegasgold:"0xffC5B358",venetianred:"0xffC80815",verdigris:"0xff43B3AE",
	vermilion:"0xffE34234",vermilion:"0xffD9381E",veronica:"0xffA020F0",verylightblue:"0xff6666FF",verylightmalachitegreen:"0xff64E986",verypaleorange:"0xffFFDFBF",
	verypaleyellow:"0xffFFFFBF",violet:"0xff8F00FF",violetcolorwheel:"0xff7F00FF",violetryb:"0xff8601AF",violet:"0xffEE82EE",violetblue:"0xff324AB2",
	violetred:"0xffF75394",viridian:"0xff40826D",viridiangreen:"0xff009698",vistablue:"0xff7C9ED9",vividauburn:"0xff922724",vividburgundy:"0xff9F1D35",
	vividcerise:"0xffDA1D81",vividgamboge:"0xffFF9900",vividmulberry:"0xffB80CE3",vividorange:"0xffFF5F00",vividorchid:"0xffCC00FF",vividraspberry:"0xffFF006C",
	vividred:"0xffF70D1A",vividredtangelo:"0xffDF6124",vividskyblue:"0xff00CCFF",vividtangelo:"0xffF07427",vividtangerine:"0xffFFA089",vividviolet:"0xff9F00FF",
	vividyellow:"0xffFFE302",warmblack:"0xff004242",waterspout:"0xffA4F4F9",wenge:"0xff645452",wheat:"0xffF5DEB3",white:"0xffFFFFFF",whitesmoke:"0xffF5F5F5",
	wildblueyonder:"0xffA2ADD0",wildorchid:"0xffD470A2",wildstrawberry:"0xffFF43A4",wildwatermelon:"0xffFC6C85",willpowerorange:"0xffFD5800",
	windsortan:"0xffA75502",wine:"0xff722F37",winedregs:"0xff673147",wisteria:"0xffC9A0DC",woodbrown:"0xffC19A6B",xanadu:"0xff738678",yaleblue:"0xff0F4D92",
	yankeesblue:"0xff1C2841",yellow:"0xffFFFF00",yellowcrayola:"0xffFCE883",yellowmunsell:"0xffEFCC00",yellowncs:"0xffFFD300",yellowpantone:"0xffFEDF00",
	yellowprocess:"0xffFFEF00",yellowryb:"0xffFEFE33",yellowgreen:"0xff9ACD32",yelloworange:"0xffFFAE42",yellowrose:"0xffFFF000",zaffre:"0xff0014A8",
	zinnwalditebrown:"0xff2C1608",zomp:"0xff39A78E", transparent:"0x00000000"
}

var fontAwesome = [
	{"unicodeEnt":"&#xf26e;","unicode":"","className":"fa-500px"},
	{"unicodeEnt":"&#xf042;","unicode":"","className":"fa-adjust"},
	{"unicodeEnt":"&#xf170;","unicode":"","className":"fa-adn"},
	{"unicodeEnt":"&#xf037;","unicode":"","className":"fa-align-center"},
	{"unicodeEnt":"&#xf039;","unicode":"","className":"fa-align-justify"},
	{"unicodeEnt":"&#xf036;","unicode":"","className":"fa-align-left"},
	{"unicodeEnt":"&#xf038;","unicode":"","className":"fa-align-right"},
	{"unicodeEnt":"&#xf270;","unicode":"","className":"fa-amazon"},
	{"unicodeEnt":"&#xf0f9;","unicode":"","className":"fa-ambulance"},
	{"unicodeEnt":"&#xf13d;","unicode":"","className":"fa-anchor"},
	{"unicodeEnt":"&#xf17b;","unicode":"","className":"fa-android"},
	{"unicodeEnt":"&#xf209;","unicode":"","className":"fa-angellist"},
	{"unicodeEnt":"&#xf103;","unicode":"","className":"fa-angle-double-down"},
	{"unicodeEnt":"&#xf100;","unicode":"","className":"fa-angle-double-left"},
	{"unicodeEnt":"&#xf101;","unicode":"","className":"fa-angle-double-right"},
	{"unicodeEnt":"&#xf102;","unicode":"","className":"fa-angle-double-up"},
	{"unicodeEnt":"&#xf107;","unicode":"","className":"fa-angle-down"},
	{"unicodeEnt":"&#xf104;","unicode":"","className":"fa-angle-left"},
	{"unicodeEnt":"&#xf105;","unicode":"","className":"fa-angle-right"},
	{"unicodeEnt":"&#xf106;","unicode":"","className":"fa-angle-up"},
	{"unicodeEnt":"&#xf179;","unicode":"","className":"fa-apple"},
	{"unicodeEnt":"&#xf187;","unicode":"","className":"fa-archive"},
	{"unicodeEnt":"&#xf1fe;","unicode":"","className":"fa-area-chart"},
	{"unicodeEnt":"&#xf0ab;","unicode":"","className":"fa-arrow-circle-down"},
	{"unicodeEnt":"&#xf0a8;","unicode":"","className":"fa-arrow-circle-left"},
	{"unicodeEnt":"&#xf01a;","unicode":"","className":"fa-arrow-circle-o-down"},
	{"unicodeEnt":"&#xf190;","unicode":"","className":"fa-arrow-circle-o-left"},
	{"unicodeEnt":"&#xf18e;","unicode":"","className":"fa-arrow-circle-o-right"},
	{"unicodeEnt":"&#xf01b;","unicode":"","className":"fa-arrow-circle-o-up"},
	{"unicodeEnt":"&#xf0a9;","unicode":"","className":"fa-arrow-circle-right"},
	{"unicodeEnt":"&#xf0aa;","unicode":"","className":"fa-arrow-circle-up"},
	{"unicodeEnt":"&#xf063;","unicode":"","className":"fa-arrow-down"},
	{"unicodeEnt":"&#xf060;","unicode":"","className":"fa-arrow-left"},
	{"unicodeEnt":"&#xf061;","unicode":"","className":"fa-arrow-right"},
	{"unicodeEnt":"&#xf062;","unicode":"","className":"fa-arrow-up"},
	{"unicodeEnt":"&#xf047;","unicode":"","className":"fa-arrows"},
	{"unicodeEnt":"&#xf0b2;","unicode":"","className":"fa-arrows-alt"},
	{"unicodeEnt":"&#xf07e;","unicode":"","className":"fa-arrows-h"},
	{"unicodeEnt":"&#xf07d;","unicode":"","className":"fa-arrows-v"},
	{"unicodeEnt":"&#xf069;","unicode":"","className":"fa-asterisk"},
	{"unicodeEnt":"&#xf1fa;","unicode":"","className":"fa-at"},
	{"unicodeEnt":"&#xf1b9;","unicode":"","className":"fa-automobile"},
	{"unicodeEnt":"&#xf04a;","unicode":"","className":"fa-backward"},
	{"unicodeEnt":"&#xf24e;","unicode":"","className":"fa-balance-scale"},
	{"unicodeEnt":"&#xf05e;","unicode":"","className":"fa-ban"},
	{"unicodeEnt":"&#xf19c;","unicode":"","className":"fa-bank"},
	{"unicodeEnt":"&#xf080;","unicode":"","className":"fa-bar-chart"},
	{"unicodeEnt":"&#xf080;","unicode":"","className":"fa-bar-chart-o"},
	{"unicodeEnt":"&#xf02a;","unicode":"","className":"fa-barcode"},
	{"unicodeEnt":"&#xf0c9;","unicode":"","className":"fa-bars"},
	{"unicodeEnt":"&#xf244;","unicode":"","className":"fa-battery-0"},
	{"unicodeEnt":"&#xf243;","unicode":"","className":"fa-battery-1"},
	{"unicodeEnt":"&#xf242;","unicode":"","className":"fa-battery-2"},
	{"unicodeEnt":"&#xf241;","unicode":"","className":"fa-battery-3"},
	{"unicodeEnt":"&#xf240;","unicode":"","className":"fa-battery-4"},
	{"unicodeEnt":"&#xf244;","unicode":"","className":"fa-battery-empty"},
	{"unicodeEnt":"&#xf240;","unicode":"","className":"fa-battery-full"},
	{"unicodeEnt":"&#xf242;","unicode":"","className":"fa-battery-half"},
	{"unicodeEnt":"&#xf243;","unicode":"","className":"fa-battery-quarter"},
	{"unicodeEnt":"&#xf241;","unicode":"","className":"fa-battery-three-quarters"},
	{"unicodeEnt":"&#xf236;","unicode":"","className":"fa-bed"},
	{"unicodeEnt":"&#xf0fc;","unicode":"","className":"fa-beer"},
	{"unicodeEnt":"&#xf1b4;","unicode":"","className":"fa-behance"},
	{"unicodeEnt":"&#xf1b5;","unicode":"","className":"fa-behance-square"},
	{"unicodeEnt":"&#xf0f3;","unicode":"","className":"fa-bell"},
	{"unicodeEnt":"&#xf0a2;","unicode":"","className":"fa-bell-o"},
	{"unicodeEnt":"&#xf1f6;","unicode":"","className":"fa-bell-slash"},
	{"unicodeEnt":"&#xf1f7;","unicode":"","className":"fa-bell-slash-o"},
	{"unicodeEnt":"&#xf206;","unicode":"","className":"fa-bicycle"},
	{"unicodeEnt":"&#xf1e5;","unicode":"","className":"fa-binoculars"},
	{"unicodeEnt":"&#xf1fd;","unicode":"","className":"fa-birthday-cake"},
	{"unicodeEnt":"&#xf171;","unicode":"","className":"fa-bitbucket"},
	{"unicodeEnt":"&#xf172;","unicode":"","className":"fa-bitbucket-square"},
	{"unicodeEnt":"&#xf15a;","unicode":"","className":"fa-bitcoin"},
	{"unicodeEnt":"&#xf27e;","unicode":"","className":"fa-black-tie"},
	{"unicodeEnt":"&#xf293;","unicode":"","className":"fa-bluetooth"},
	{"unicodeEnt":"&#xf294;","unicode":"","className":"fa-bluetooth-b"},
	{"unicodeEnt":"&#xf032;","unicode":"","className":"fa-bold"},
	{"unicodeEnt":"&#xf0e7;","unicode":"","className":"fa-bolt"},
	{"unicodeEnt":"&#xf1e2;","unicode":"","className":"fa-bomb"},
	{"unicodeEnt":"&#xf02d;","unicode":"","className":"fa-book"},
	{"unicodeEnt":"&#xf02e;","unicode":"","className":"fa-bookmark"},
	{"unicodeEnt":"&#xf097;","unicode":"","className":"fa-bookmark-o"},
	{"unicodeEnt":"&#xf0b1;","unicode":"","className":"fa-briefcase"},
	{"unicodeEnt":"&#xf15a;","unicode":"","className":"fa-btc"},
	{"unicodeEnt":"&#xf188;","unicode":"","className":"fa-bug"},
	{"unicodeEnt":"&#xf1ad;","unicode":"","className":"fa-building"},
	{"unicodeEnt":"&#xf0f7;","unicode":"","className":"fa-building-o"},
	{"unicodeEnt":"&#xf0a1;","unicode":"","className":"fa-bullhorn"},
	{"unicodeEnt":"&#xf140;","unicode":"","className":"fa-bullseye"},
	{"unicodeEnt":"&#xf207;","unicode":"","className":"fa-bus"},
	{"unicodeEnt":"&#xf20d;","unicode":"","className":"fa-buysellads"},
	{"unicodeEnt":"&#xf1ba;","unicode":"","className":"fa-cab"},
	{"unicodeEnt":"&#xf1ec;","unicode":"","className":"fa-calculator"},
	{"unicodeEnt":"&#xf073;","unicode":"","className":"fa-calendar"},
	{"unicodeEnt":"&#xf274;","unicode":"","className":"fa-calendar-check-o"},
	{"unicodeEnt":"&#xf272;","unicode":"","className":"fa-calendar-minus-o"},
	{"unicodeEnt":"&#xf133;","unicode":"","className":"fa-calendar-o"},
	{"unicodeEnt":"&#xf271;","unicode":"","className":"fa-calendar-plus-o"},
	{"unicodeEnt":"&#xf273;","unicode":"","className":"fa-calendar-times-o"},
	{"unicodeEnt":"&#xf030;","unicode":"","className":"fa-camera"},
	{"unicodeEnt":"&#xf083;","unicode":"","className":"fa-camera-retro"},
	{"unicodeEnt":"&#xf1b9;","unicode":"","className":"fa-car"},
	{"unicodeEnt":"&#xf0d7;","unicode":"","className":"fa-caret-down"},
	{"unicodeEnt":"&#xf0d9;","unicode":"","className":"fa-caret-left"},
	{"unicodeEnt":"&#xf0da;","unicode":"","className":"fa-caret-right"},
	{"unicodeEnt":"&#xf150;","unicode":"","className":"fa-caret-square-o-down"},
	{"unicodeEnt":"&#xf191;","unicode":"","className":"fa-caret-square-o-left"},
	{"unicodeEnt":"&#xf152;","unicode":"","className":"fa-caret-square-o-right"},
	{"unicodeEnt":"&#xf151;","unicode":"","className":"fa-caret-square-o-up"},
	{"unicodeEnt":"&#xf0d8;","unicode":"","className":"fa-caret-up"},
	{"unicodeEnt":"&#xf218;","unicode":"","className":"fa-cart-arrow-down"},
	{"unicodeEnt":"&#xf217;","unicode":"","className":"fa-cart-plus"},
	{"unicodeEnt":"&#xf20a;","unicode":"","className":"fa-cc"},
	{"unicodeEnt":"&#xf1f3;","unicode":"","className":"fa-cc-amex"},
	{"unicodeEnt":"&#xf24c;","unicode":"","className":"fa-cc-diners-club"},
	{"unicodeEnt":"&#xf1f2;","unicode":"","className":"fa-cc-discover"},
	{"unicodeEnt":"&#xf24b;","unicode":"","className":"fa-cc-jcb"},
	{"unicodeEnt":"&#xf1f1;","unicode":"","className":"fa-cc-mastercard"},
	{"unicodeEnt":"&#xf1f4;","unicode":"","className":"fa-cc-paypal"},
	{"unicodeEnt":"&#xf1f5;","unicode":"","className":"fa-cc-stripe"},
	{"unicodeEnt":"&#xf1f0;","unicode":"","className":"fa-cc-visa"},
	{"unicodeEnt":"&#xf0a3;","unicode":"","className":"fa-certificate"},
	{"unicodeEnt":"&#xf0c1;","unicode":"","className":"fa-chain"},
	{"unicodeEnt":"&#xf127;","unicode":"","className":"fa-chain-broken"},
	{"unicodeEnt":"&#xf00c;","unicode":"","className":"fa-check"},
	{"unicodeEnt":"&#xf058;","unicode":"","className":"fa-check-circle"},
	{"unicodeEnt":"&#xf05d;","unicode":"","className":"fa-check-circle-o"},
	{"unicodeEnt":"&#xf14a;","unicode":"","className":"fa-check-square"},
	{"unicodeEnt":"&#xf046;","unicode":"","className":"fa-check-square-o"},
	{"unicodeEnt":"&#xf13a;","unicode":"","className":"fa-chevron-circle-down"},
	{"unicodeEnt":"&#xf137;","unicode":"","className":"fa-chevron-circle-left"},
	{"unicodeEnt":"&#xf138;","unicode":"","className":"fa-chevron-circle-right"},
	{"unicodeEnt":"&#xf139;","unicode":"","className":"fa-chevron-circle-up"},
	{"unicodeEnt":"&#xf078;","unicode":"","className":"fa-chevron-down"},
	{"unicodeEnt":"&#xf053;","unicode":"","className":"fa-chevron-left"},
	{"unicodeEnt":"&#xf054;","unicode":"","className":"fa-chevron-right"},
	{"unicodeEnt":"&#xf077;","unicode":"","className":"fa-chevron-up"},
	{"unicodeEnt":"&#xf1ae;","unicode":"","className":"fa-child"},
	{"unicodeEnt":"&#xf268;","unicode":"","className":"fa-chrome"},
	{"unicodeEnt":"&#xf111;","unicode":"","className":"fa-circle"},
	{"unicodeEnt":"&#xf10c;","unicode":"","className":"fa-circle-o"},
	{"unicodeEnt":"&#xf1ce;","unicode":"","className":"fa-circle-o-notch"},
	{"unicodeEnt":"&#xf1db;","unicode":"","className":"fa-circle-thin"},
	{"unicodeEnt":"&#xf0ea;","unicode":"","className":"fa-clipboard"},
	{"unicodeEnt":"&#xf017;","unicode":"","className":"fa-clock-o"},
	{"unicodeEnt":"&#xf24d;","unicode":"","className":"fa-clone"},
	{"unicodeEnt":"&#xf00d;","unicode":"","className":"fa-close"},
	{"unicodeEnt":"&#xf0c2;","unicode":"","className":"fa-cloud"},
	{"unicodeEnt":"&#xf0ed;","unicode":"","className":"fa-cloud-download"},
	{"unicodeEnt":"&#xf0ee;","unicode":"","className":"fa-cloud-upload"},
	{"unicodeEnt":"&#xf157;","unicode":"","className":"fa-cny"},
	{"unicodeEnt":"&#xf121;","unicode":"","className":"fa-code"},
	{"unicodeEnt":"&#xf126;","unicode":"","className":"fa-code-fork"},
	{"unicodeEnt":"&#xf1cb;","unicode":"","className":"fa-codepen"},
	{"unicodeEnt":"&#xf284;","unicode":"","className":"fa-codiepie"},
	{"unicodeEnt":"&#xf0f4;","unicode":"","className":"fa-coffee"},
	{"unicodeEnt":"&#xf013;","unicode":"","className":"fa-cog"},
	{"unicodeEnt":"&#xf085;","unicode":"","className":"fa-cogs"},
	{"unicodeEnt":"&#xf0db;","unicode":"","className":"fa-columns"},
	{"unicodeEnt":"&#xf075;","unicode":"","className":"fa-comment"},
	{"unicodeEnt":"&#xf0e5;","unicode":"","className":"fa-comment-o"},
	{"unicodeEnt":"&#xf27a;","unicode":"","className":"fa-commenting"},
	{"unicodeEnt":"&#xf27b;","unicode":"","className":"fa-commenting-o"},
	{"unicodeEnt":"&#xf086;","unicode":"","className":"fa-comments"},
	{"unicodeEnt":"&#xf0e6;","unicode":"","className":"fa-comments-o"},
	{"unicodeEnt":"&#xf14e;","unicode":"","className":"fa-compass"},
	{"unicodeEnt":"&#xf066;","unicode":"","className":"fa-compress"},
	{"unicodeEnt":"&#xf20e;","unicode":"","className":"fa-connectdevelop"},
	{"unicodeEnt":"&#xf26d;","unicode":"","className":"fa-contao"},
	{"unicodeEnt":"&#xf0c5;","unicode":"","className":"fa-copy"},
	{"unicodeEnt":"&#xf1f9;","unicode":"","className":"fa-copyright"},
	{"unicodeEnt":"&#xf25e;","unicode":"","className":"fa-creative-commons"},
	{"unicodeEnt":"&#xf09d;","unicode":"","className":"fa-credit-card"},
	{"unicodeEnt":"&#xf283;","unicode":"","className":"fa-credit-card-alt"},
	{"unicodeEnt":"&#xf125;","unicode":"","className":"fa-crop"},
	{"unicodeEnt":"&#xf05b;","unicode":"","className":"fa-crosshairs"},
	{"unicodeEnt":"&#xf13c;","unicode":"","className":"fa-css3"},
	{"unicodeEnt":"&#xf1b2;","unicode":"","className":"fa-cube"},
	{"unicodeEnt":"&#xf1b3;","unicode":"","className":"fa-cubes"},
	{"unicodeEnt":"&#xf0c4;","unicode":"","className":"fa-cut"},
	{"unicodeEnt":"&#xf0f5;","unicode":"","className":"fa-cutlery"},
	{"unicodeEnt":"&#xf0e4;","unicode":"","className":"fa-dashboard"},
	{"unicodeEnt":"&#xf210;","unicode":"","className":"fa-dashcube"},
	{"unicodeEnt":"&#xf1c0;","unicode":"","className":"fa-database"},
	{"unicodeEnt":"&#xf03b;","unicode":"","className":"fa-dedent"},
	{"unicodeEnt":"&#xf1a5;","unicode":"","className":"fa-delicious"},
	{"unicodeEnt":"&#xf108;","unicode":"","className":"fa-desktop"},
	{"unicodeEnt":"&#xf1bd;","unicode":"","className":"fa-deviantart"},
	{"unicodeEnt":"&#xf219;","unicode":"","className":"fa-diamond"},
	{"unicodeEnt":"&#xf1a6;","unicode":"","className":"fa-digg"},
	{"unicodeEnt":"&#xf155;","unicode":"","className":"fa-dollar"},
	{"unicodeEnt":"&#xf192;","unicode":"","className":"fa-dot-circle-o"},
	{"unicodeEnt":"&#xf019;","unicode":"","className":"fa-download"},
	{"unicodeEnt":"&#xf17d;","unicode":"","className":"fa-dribbble"},
	{"unicodeEnt":"&#xf16b;","unicode":"","className":"fa-dropbox"},
	{"unicodeEnt":"&#xf1a9;","unicode":"","className":"fa-drupal"},
	{"unicodeEnt":"&#xf282;","unicode":"","className":"fa-edge"},
	{"unicodeEnt":"&#xf044;","unicode":"","className":"fa-edit"},
	{"unicodeEnt":"&#xf052;","unicode":"","className":"fa-eject"},
	{"unicodeEnt":"&#xf141;","unicode":"","className":"fa-ellipsis-h"},
	{"unicodeEnt":"&#xf142;","unicode":"","className":"fa-ellipsis-v"},
	{"unicodeEnt":"&#xf1d1;","unicode":"","className":"fa-empire"},
	{"unicodeEnt":"&#xf0e0;","unicode":"","className":"fa-envelope"},
	{"unicodeEnt":"&#xf003;","unicode":"","className":"fa-envelope-o"},
	{"unicodeEnt":"&#xf199;","unicode":"","className":"fa-envelope-square"},
	{"unicodeEnt":"&#xf12d;","unicode":"","className":"fa-eraser"},
	{"unicodeEnt":"&#xf153;","unicode":"","className":"fa-eur"},
	{"unicodeEnt":"&#xf153;","unicode":"","className":"fa-euro"},
	{"unicodeEnt":"&#xf0ec;","unicode":"","className":"fa-exchange"},
	{"unicodeEnt":"&#xf12a;","unicode":"","className":"fa-exclamation"},
	{"unicodeEnt":"&#xf06a;","unicode":"","className":"fa-exclamation-circle"},
	{"unicodeEnt":"&#xf071;","unicode":"","className":"fa-exclamation-triangle"},
	{"unicodeEnt":"&#xf065;","unicode":"","className":"fa-expand"},
	{"unicodeEnt":"&#xf23e;","unicode":"","className":"fa-expeditedssl"},
	{"unicodeEnt":"&#xf08e;","unicode":"","className":"fa-external-link"},
	{"unicodeEnt":"&#xf14c;","unicode":"","className":"fa-external-link-square"},
	{"unicodeEnt":"&#xf06e;","unicode":"","className":"fa-eye"},
	{"unicodeEnt":"&#xf070;","unicode":"","className":"fa-eye-slash"},
	{"unicodeEnt":"&#xf1fb;","unicode":"","className":"fa-eyedropper"},
	{"unicodeEnt":"&#xf09a;","unicode":"","className":"fa-facebook"},
	{"unicodeEnt":"&#xf09a;","unicode":"","className":"fa-facebook-f"},
	{"unicodeEnt":"&#xf230;","unicode":"","className":"fa-facebook-official"},
	{"unicodeEnt":"&#xf082;","unicode":"","className":"fa-facebook-square"},
	{"unicodeEnt":"&#xf049;","unicode":"","className":"fa-fast-backward"},
	{"unicodeEnt":"&#xf050;","unicode":"","className":"fa-fast-forward"},
	{"unicodeEnt":"&#xf1ac;","unicode":"","className":"fa-fax"},
	{"unicodeEnt":"&#xf09e;","unicode":"","className":"fa-feed"},
	{"unicodeEnt":"&#xf182;","unicode":"","className":"fa-female"},
	{"unicodeEnt":"&#xf0fb;","unicode":"","className":"fa-fighter-jet"},
	{"unicodeEnt":"&#xf15b;","unicode":"","className":"fa-file"},
	{"unicodeEnt":"&#xf1c6;","unicode":"","className":"fa-file-archive-o"},
	{"unicodeEnt":"&#xf1c7;","unicode":"","className":"fa-file-audio-o"},
	{"unicodeEnt":"&#xf1c9;","unicode":"","className":"fa-file-code-o"},
	{"unicodeEnt":"&#xf1c3;","unicode":"","className":"fa-file-excel-o"},
	{"unicodeEnt":"&#xf1c5;","unicode":"","className":"fa-file-image-o"},
	{"unicodeEnt":"&#xf1c8;","unicode":"","className":"fa-file-movie-o"},
	{"unicodeEnt":"&#xf016;","unicode":"","className":"fa-file-o"},
	{"unicodeEnt":"&#xf1c1;","unicode":"","className":"fa-file-pdf-o"},
	{"unicodeEnt":"&#xf1c5;","unicode":"","className":"fa-file-photo-o"},
	{"unicodeEnt":"&#xf1c5;","unicode":"","className":"fa-file-picture-o"},
	{"unicodeEnt":"&#xf1c4;","unicode":"","className":"fa-file-powerpoint-o"},
	{"unicodeEnt":"&#xf1c7;","unicode":"","className":"fa-file-sound-o"},
	{"unicodeEnt":"&#xf15c;","unicode":"","className":"fa-file-text"},
	{"unicodeEnt":"&#xf0f6;","unicode":"","className":"fa-file-text-o"},
	{"unicodeEnt":"&#xf1c8;","unicode":"","className":"fa-file-video-o"},
	{"unicodeEnt":"&#xf1c2;","unicode":"","className":"fa-file-word-o"},
	{"unicodeEnt":"&#xf1c6;","unicode":"","className":"fa-file-zip-o"},
	{"unicodeEnt":"&#xf0c5;","unicode":"","className":"fa-files-o"},
	{"unicodeEnt":"&#xf008;","unicode":"","className":"fa-film"},
	{"unicodeEnt":"&#xf0b0;","unicode":"","className":"fa-filter"},
	{"unicodeEnt":"&#xf06d;","unicode":"","className":"fa-fire"},
	{"unicodeEnt":"&#xf134;","unicode":"","className":"fa-fire-extinguisher"},
	{"unicodeEnt":"&#xf269;","unicode":"","className":"fa-firefox"},
	{"unicodeEnt":"&#xf024;","unicode":"","className":"fa-flag"},
	{"unicodeEnt":"&#xf11e;","unicode":"","className":"fa-flag-checkered"},
	{"unicodeEnt":"&#xf11d;","unicode":"","className":"fa-flag-o"},
	{"unicodeEnt":"&#xf0e7;","unicode":"","className":"fa-flash"},
	{"unicodeEnt":"&#xf0c3;","unicode":"","className":"fa-flask"},
	{"unicodeEnt":"&#xf16e;","unicode":"","className":"fa-flickr"},
	{"unicodeEnt":"&#xf0c7;","unicode":"","className":"fa-floppy-o"},
	{"unicodeEnt":"&#xf07b;","unicode":"","className":"fa-folder"},
	{"unicodeEnt":"&#xf114;","unicode":"","className":"fa-folder-o"},
	{"unicodeEnt":"&#xf07c;","unicode":"","className":"fa-folder-open"},
	{"unicodeEnt":"&#xf115;","unicode":"","className":"fa-folder-open-o"},
	{"unicodeEnt":"&#xf031;","unicode":"","className":"fa-font"},
	{"unicodeEnt":"&#xf280;","unicode":"","className":"fa-fonticons"},
	{"unicodeEnt":"&#xf286;","unicode":"","className":"fa-fort-awesome"},
	{"unicodeEnt":"&#xf211;","unicode":"","className":"fa-forumbee"},
	{"unicodeEnt":"&#xf04e;","unicode":"","className":"fa-forward"},
	{"unicodeEnt":"&#xf180;","unicode":"","className":"fa-foursquare"},
	{"unicodeEnt":"&#xf119;","unicode":"","className":"fa-frown-o"},
	{"unicodeEnt":"&#xf1e3;","unicode":"","className":"fa-futbol-o"},
	{"unicodeEnt":"&#xf11b;","unicode":"","className":"fa-gamepad"},
	{"unicodeEnt":"&#xf0e3;","unicode":"","className":"fa-gavel"},
	{"unicodeEnt":"&#xf154;","unicode":"","className":"fa-gbp"},
	{"unicodeEnt":"&#xf1d1;","unicode":"","className":"fa-ge"},
	{"unicodeEnt":"&#xf013;","unicode":"","className":"fa-gear"},
	{"unicodeEnt":"&#xf085;","unicode":"","className":"fa-gears"},
	{"unicodeEnt":"&#xf22d;","unicode":"","className":"fa-genderless"},
	{"unicodeEnt":"&#xf265;","unicode":"","className":"fa-get-pocket"},
	{"unicodeEnt":"&#xf260;","unicode":"","className":"fa-gg"},
	{"unicodeEnt":"&#xf261;","unicode":"","className":"fa-gg-circle"},
	{"unicodeEnt":"&#xf06b;","unicode":"","className":"fa-gift"},
	{"unicodeEnt":"&#xf1d3;","unicode":"","className":"fa-git"},
	{"unicodeEnt":"&#xf1d2;","unicode":"","className":"fa-git-square"},
	{"unicodeEnt":"&#xf09b;","unicode":"","className":"fa-github"},
	{"unicodeEnt":"&#xf113;","unicode":"","className":"fa-github-alt"},
	{"unicodeEnt":"&#xf092;","unicode":"","className":"fa-github-square"},
	{"unicodeEnt":"&#xf184;","unicode":"","className":"fa-gittip"},
	{"unicodeEnt":"&#xf000;","unicode":"","className":"fa-glass"},
	{"unicodeEnt":"&#xf0ac;","unicode":"","className":"fa-globe"},
	{"unicodeEnt":"&#xf1a0;","unicode":"","className":"fa-google"},
	{"unicodeEnt":"&#xf0d5;","unicode":"","className":"fa-google-plus"},
	{"unicodeEnt":"&#xf0d4;","unicode":"","className":"fa-google-plus-square"},
	{"unicodeEnt":"&#xf1ee;","unicode":"","className":"fa-google-wallet"},
	{"unicodeEnt":"&#xf19d;","unicode":"","className":"fa-graduation-cap"},
	{"unicodeEnt":"&#xf184;","unicode":"","className":"fa-gratipay"},
	{"unicodeEnt":"&#xf0c0;","unicode":"","className":"fa-group"},
	{"unicodeEnt":"&#xf0fd;","unicode":"","className":"fa-h-square"},
	{"unicodeEnt":"&#xf1d4;","unicode":"","className":"fa-hacker-news"},
	{"unicodeEnt":"&#xf255;","unicode":"","className":"fa-hand-grab-o"},
	{"unicodeEnt":"&#xf258;","unicode":"","className":"fa-hand-lizard-o"},
	{"unicodeEnt":"&#xf0a7;","unicode":"","className":"fa-hand-o-down"},
	{"unicodeEnt":"&#xf0a5;","unicode":"","className":"fa-hand-o-left"},
	{"unicodeEnt":"&#xf0a4;","unicode":"","className":"fa-hand-o-right"},
	{"unicodeEnt":"&#xf0a6;","unicode":"","className":"fa-hand-o-up"},
	{"unicodeEnt":"&#xf256;","unicode":"","className":"fa-hand-paper-o"},
	{"unicodeEnt":"&#xf25b;","unicode":"","className":"fa-hand-peace-o"},
	{"unicodeEnt":"&#xf25a;","unicode":"","className":"fa-hand-pointer-o"},
	{"unicodeEnt":"&#xf255;","unicode":"","className":"fa-hand-rock-o"},
	{"unicodeEnt":"&#xf257;","unicode":"","className":"fa-hand-scissors-o"},
	{"unicodeEnt":"&#xf259;","unicode":"","className":"fa-hand-spock-o"},
	{"unicodeEnt":"&#xf256;","unicode":"","className":"fa-hand-stop-o"},
	{"unicodeEnt":"&#xf292;","unicode":"","className":"fa-hashtag"},
	{"unicodeEnt":"&#xf0a0;","unicode":"","className":"fa-hdd-o"},
	{"unicodeEnt":"&#xf1dc;","unicode":"","className":"fa-header"},
	{"unicodeEnt":"&#xf025;","unicode":"","className":"fa-headphones"},
	{"unicodeEnt":"&#xf004;","unicode":"","className":"fa-heart"},
	{"unicodeEnt":"&#xf08a;","unicode":"","className":"fa-heart-o"},
	{"unicodeEnt":"&#xf21e;","unicode":"","className":"fa-heartbeat"},
	{"unicodeEnt":"&#xf1da;","unicode":"","className":"fa-history"},
	{"unicodeEnt":"&#xf015;","unicode":"","className":"fa-home"},
	{"unicodeEnt":"&#xf0f8;","unicode":"","className":"fa-hospital-o"},
	{"unicodeEnt":"&#xf236;","unicode":"","className":"fa-hotel"},
	{"unicodeEnt":"&#xf254;","unicode":"","className":"fa-hourglass"},
	{"unicodeEnt":"&#xf251;","unicode":"","className":"fa-hourglass-1"},
	{"unicodeEnt":"&#xf252;","unicode":"","className":"fa-hourglass-2"},
	{"unicodeEnt":"&#xf253;","unicode":"","className":"fa-hourglass-3"},
	{"unicodeEnt":"&#xf253;","unicode":"","className":"fa-hourglass-end"},
	{"unicodeEnt":"&#xf252;","unicode":"","className":"fa-hourglass-half"},
	{"unicodeEnt":"&#xf250;","unicode":"","className":"fa-hourglass-o"},
	{"unicodeEnt":"&#xf251;","unicode":"","className":"fa-hourglass-start"},
	{"unicodeEnt":"&#xf27c;","unicode":"","className":"fa-houzz"},
	{"unicodeEnt":"&#xf13b;","unicode":"","className":"fa-html5"},
	{"unicodeEnt":"&#xf246;","unicode":"","className":"fa-i-cursor"},
	{"unicodeEnt":"&#xf20b;","unicode":"","className":"fa-ils"},
	{"unicodeEnt":"&#xf03e;","unicode":"","className":"fa-image"},
	{"unicodeEnt":"&#xf01c;","unicode":"","className":"fa-inbox"},
	{"unicodeEnt":"&#xf03c;","unicode":"","className":"fa-indent"},
	{"unicodeEnt":"&#xf275;","unicode":"","className":"fa-industry"},
	{"unicodeEnt":"&#xf129;","unicode":"","className":"fa-info"},
	{"unicodeEnt":"&#xf05a;","unicode":"","className":"fa-info-circle"},
	{"unicodeEnt":"&#xf156;","unicode":"","className":"fa-inr"},
	{"unicodeEnt":"&#xf16d;","unicode":"","className":"fa-instagram"},
	{"unicodeEnt":"&#xf19c;","unicode":"","className":"fa-institution"},
	{"unicodeEnt":"&#xf26b;","unicode":"","className":"fa-internet-explorer"},
	{"unicodeEnt":"&#xf224;","unicode":"","className":"fa-intersex"},
	{"unicodeEnt":"&#xf208;","unicode":"","className":"fa-ioxhost"},
	{"unicodeEnt":"&#xf033;","unicode":"","className":"fa-italic"},
	{"unicodeEnt":"&#xf1aa;","unicode":"","className":"fa-joomla"},
	{"unicodeEnt":"&#xf157;","unicode":"","className":"fa-jpy"},
	{"unicodeEnt":"&#xf1cc;","unicode":"","className":"fa-jsfiddle"},
	{"unicodeEnt":"&#xf084;","unicode":"","className":"fa-key"},
	{"unicodeEnt":"&#xf11c;","unicode":"","className":"fa-keyboard-o"},
	{"unicodeEnt":"&#xf159;","unicode":"","className":"fa-krw"},
	{"unicodeEnt":"&#xf1ab;","unicode":"","className":"fa-language"},
	{"unicodeEnt":"&#xf109;","unicode":"","className":"fa-laptop"},
	{"unicodeEnt":"&#xf202;","unicode":"","className":"fa-lastfm"},
	{"unicodeEnt":"&#xf203;","unicode":"","className":"fa-lastfm-square"},
	{"unicodeEnt":"&#xf06c;","unicode":"","className":"fa-leaf"},
	{"unicodeEnt":"&#xf212;","unicode":"","className":"fa-leanpub"},
	{"unicodeEnt":"&#xf0e3;","unicode":"","className":"fa-legal"},
	{"unicodeEnt":"&#xf094;","unicode":"","className":"fa-lemon-o"},
	{"unicodeEnt":"&#xf149;","unicode":"","className":"fa-level-down"},
	{"unicodeEnt":"&#xf148;","unicode":"","className":"fa-level-up"},
	{"unicodeEnt":"&#xf1cd;","unicode":"","className":"fa-life-bouy"},
	{"unicodeEnt":"&#xf1cd;","unicode":"","className":"fa-life-buoy"},
	{"unicodeEnt":"&#xf1cd;","unicode":"","className":"fa-life-ring"},
	{"unicodeEnt":"&#xf1cd;","unicode":"","className":"fa-life-saver"},
	{"unicodeEnt":"&#xf0eb;","unicode":"","className":"fa-lightbulb-o"},
	{"unicodeEnt":"&#xf201;","unicode":"","className":"fa-line-chart"},
	{"unicodeEnt":"&#xf0c1;","unicode":"","className":"fa-link"},
	{"unicodeEnt":"&#xf0e1;","unicode":"","className":"fa-linkedin"},
	{"unicodeEnt":"&#xf08c;","unicode":"","className":"fa-linkedin-square"},
	{"unicodeEnt":"&#xf17c;","unicode":"","className":"fa-linux"},
	{"unicodeEnt":"&#xf03a;","unicode":"","className":"fa-list"},
	{"unicodeEnt":"&#xf022;","unicode":"","className":"fa-list-alt"},
	{"unicodeEnt":"&#xf0cb;","unicode":"","className":"fa-list-ol"},
	{"unicodeEnt":"&#xf0ca;","unicode":"","className":"fa-list-ul"},
	{"unicodeEnt":"&#xf124;","unicode":"","className":"fa-location-arrow"},
	{"unicodeEnt":"&#xf023;","unicode":"","className":"fa-lock"},
	{"unicodeEnt":"&#xf175;","unicode":"","className":"fa-long-arrow-down"},
	{"unicodeEnt":"&#xf177;","unicode":"","className":"fa-long-arrow-left"},
	{"unicodeEnt":"&#xf178;","unicode":"","className":"fa-long-arrow-right"},
	{"unicodeEnt":"&#xf176;","unicode":"","className":"fa-long-arrow-up"},
	{"unicodeEnt":"&#xf0d0;","unicode":"","className":"fa-magic"},
	{"unicodeEnt":"&#xf076;","unicode":"","className":"fa-magnet"},
	{"unicodeEnt":"&#xf064;","unicode":"","className":"fa-mail-forward"},
	{"unicodeEnt":"&#xf112;","unicode":"","className":"fa-mail-reply"},
	{"unicodeEnt":"&#xf122;","unicode":"","className":"fa-mail-reply-all"},
	{"unicodeEnt":"&#xf183;","unicode":"","className":"fa-male"},
	{"unicodeEnt":"&#xf279;","unicode":"","className":"fa-map"},
	{"unicodeEnt":"&#xf041;","unicode":"","className":"fa-map-marker"},
	{"unicodeEnt":"&#xf278;","unicode":"","className":"fa-map-o"},
	{"unicodeEnt":"&#xf276;","unicode":"","className":"fa-map-pin"},
	{"unicodeEnt":"&#xf277;","unicode":"","className":"fa-map-signs"},
	{"unicodeEnt":"&#xf222;","unicode":"","className":"fa-mars"},
	{"unicodeEnt":"&#xf227;","unicode":"","className":"fa-mars-double"},
	{"unicodeEnt":"&#xf229;","unicode":"","className":"fa-mars-stroke"},
	{"unicodeEnt":"&#xf22b;","unicode":"","className":"fa-mars-stroke-h"},
	{"unicodeEnt":"&#xf22a;","unicode":"","className":"fa-mars-stroke-v"},
	{"unicodeEnt":"&#xf136;","unicode":"","className":"fa-maxcdn"},
	{"unicodeEnt":"&#xf20c;","unicode":"","className":"fa-meanpath"},
	{"unicodeEnt":"&#xf23a;","unicode":"","className":"fa-medium"},
	{"unicodeEnt":"&#xf0fa;","unicode":"","className":"fa-medkit"},
	{"unicodeEnt":"&#xf11a;","unicode":"","className":"fa-meh-o"},
	{"unicodeEnt":"&#xf223;","unicode":"","className":"fa-mercury"},
	{"unicodeEnt":"&#xf130;","unicode":"","className":"fa-microphone"},
	{"unicodeEnt":"&#xf131;","unicode":"","className":"fa-microphone-slash"},
	{"unicodeEnt":"&#xf068;","unicode":"","className":"fa-minus"},
	{"unicodeEnt":"&#xf056;","unicode":"","className":"fa-minus-circle"},
	{"unicodeEnt":"&#xf146;","unicode":"","className":"fa-minus-square"},
	{"unicodeEnt":"&#xf147;","unicode":"","className":"fa-minus-square-o"},
	{"unicodeEnt":"&#xf289;","unicode":"","className":"fa-mixcloud"},
	{"unicodeEnt":"&#xf10b;","unicode":"","className":"fa-mobile"},
	{"unicodeEnt":"&#xf10b;","unicode":"","className":"fa-mobile-phone"},
	{"unicodeEnt":"&#xf285;","unicode":"","className":"fa-modx"},
	{"unicodeEnt":"&#xf0d6;","unicode":"","className":"fa-money"},
	{"unicodeEnt":"&#xf186;","unicode":"","className":"fa-moon-o"},
	{"unicodeEnt":"&#xf19d;","unicode":"","className":"fa-mortar-board"},
	{"unicodeEnt":"&#xf21c;","unicode":"","className":"fa-motorcycle"},
	{"unicodeEnt":"&#xf245;","unicode":"","className":"fa-mouse-pointer"},
	{"unicodeEnt":"&#xf001;","unicode":"","className":"fa-music"},
	{"unicodeEnt":"&#xf0c9;","unicode":"","className":"fa-navicon"},
	{"unicodeEnt":"&#xf22c;","unicode":"","className":"fa-neuter"},
	{"unicodeEnt":"&#xf1ea;","unicode":"","className":"fa-newspaper-o"},
	{"unicodeEnt":"&#xf247;","unicode":"","className":"fa-object-group"},
	{"unicodeEnt":"&#xf248;","unicode":"","className":"fa-object-ungroup"},
	{"unicodeEnt":"&#xf263;","unicode":"","className":"fa-odnoklassniki"},
	{"unicodeEnt":"&#xf264;","unicode":"","className":"fa-odnoklassniki-square"},
	{"unicodeEnt":"&#xf23d;","unicode":"","className":"fa-opencart"},
	{"unicodeEnt":"&#xf19b;","unicode":"","className":"fa-openid"},
	{"unicodeEnt":"&#xf26a;","unicode":"","className":"fa-opera"},
	{"unicodeEnt":"&#xf23c;","unicode":"","className":"fa-optin-monster"},
	{"unicodeEnt":"&#xf03b;","unicode":"","className":"fa-outdent"},
	{"unicodeEnt":"&#xf18c;","unicode":"","className":"fa-pagelines"},
	{"unicodeEnt":"&#xf1fc;","unicode":"","className":"fa-paint-brush"},
	{"unicodeEnt":"&#xf1d8;","unicode":"","className":"fa-paper-plane"},
	{"unicodeEnt":"&#xf1d9;","unicode":"","className":"fa-paper-plane-o"},
	{"unicodeEnt":"&#xf0c6;","unicode":"","className":"fa-paperclip"},
	{"unicodeEnt":"&#xf1dd;","unicode":"","className":"fa-paragraph"},
	{"unicodeEnt":"&#xf0ea;","unicode":"","className":"fa-paste"},
	{"unicodeEnt":"&#xf04c;","unicode":"","className":"fa-pause"},
	{"unicodeEnt":"&#xf28b;","unicode":"","className":"fa-pause-circle"},
	{"unicodeEnt":"&#xf28c;","unicode":"","className":"fa-pause-circle-o"},
	{"unicodeEnt":"&#xf1b0;","unicode":"","className":"fa-paw"},
	{"unicodeEnt":"&#xf1ed;","unicode":"","className":"fa-paypal"},
	{"unicodeEnt":"&#xf040;","unicode":"","className":"fa-pencil"},
	{"unicodeEnt":"&#xf14b;","unicode":"","className":"fa-pencil-square"},
	{"unicodeEnt":"&#xf044;","unicode":"","className":"fa-pencil-square-o"},
	{"unicodeEnt":"&#xf295;","unicode":"","className":"fa-percent"},
	{"unicodeEnt":"&#xf095;","unicode":"","className":"fa-phone"},
	{"unicodeEnt":"&#xf098;","unicode":"","className":"fa-phone-square"},
	{"unicodeEnt":"&#xf03e;","unicode":"","className":"fa-photo"},
	{"unicodeEnt":"&#xf03e;","unicode":"","className":"fa-picture-o"},
	{"unicodeEnt":"&#xf200;","unicode":"","className":"fa-pie-chart"},
	{"unicodeEnt":"&#xf1a7;","unicode":"","className":"fa-pied-piper"},
	{"unicodeEnt":"&#xf1a8;","unicode":"","className":"fa-pied-piper-alt"},
	{"unicodeEnt":"&#xf0d2;","unicode":"","className":"fa-pinterest"},
	{"unicodeEnt":"&#xf231;","unicode":"","className":"fa-pinterest-p"},
	{"unicodeEnt":"&#xf0d3;","unicode":"","className":"fa-pinterest-square"},
	{"unicodeEnt":"&#xf072;","unicode":"","className":"fa-plane"},
	{"unicodeEnt":"&#xf04b;","unicode":"","className":"fa-play"},
	{"unicodeEnt":"&#xf144;","unicode":"","className":"fa-play-circle"},
	{"unicodeEnt":"&#xf01d;","unicode":"","className":"fa-play-circle-o"},
	{"unicodeEnt":"&#xf1e6;","unicode":"","className":"fa-plug"},
	{"unicodeEnt":"&#xf067;","unicode":"","className":"fa-plus"},
	{"unicodeEnt":"&#xf055;","unicode":"","className":"fa-plus-circle"},
	{"unicodeEnt":"&#xf0fe;","unicode":"","className":"fa-plus-square"},
	{"unicodeEnt":"&#xf196;","unicode":"","className":"fa-plus-square-o"},
	{"unicodeEnt":"&#xf011;","unicode":"","className":"fa-power-off"},
	{"unicodeEnt":"&#xf02f;","unicode":"","className":"fa-print"},
	{"unicodeEnt":"&#xf288;","unicode":"","className":"fa-product-hunt"},
	{"unicodeEnt":"&#xf12e;","unicode":"","className":"fa-puzzle-piece"},
	{"unicodeEnt":"&#xf1d6;","unicode":"","className":"fa-qq"},
	{"unicodeEnt":"&#xf029;","unicode":"","className":"fa-qrcode"},
	{"unicodeEnt":"&#xf128;","unicode":"","className":"fa-question"},
	{"unicodeEnt":"&#xf059;","unicode":"","className":"fa-question-circle"},
	{"unicodeEnt":"&#xf10d;","unicode":"","className":"fa-quote-left"},
	{"unicodeEnt":"&#xf10e;","unicode":"","className":"fa-quote-right"},
	{"unicodeEnt":"&#xf1d0;","unicode":"","className":"fa-ra"},
	{"unicodeEnt":"&#xf074;","unicode":"","className":"fa-random"},
	{"unicodeEnt":"&#xf1d0;","unicode":"","className":"fa-rebel"},
	{"unicodeEnt":"&#xf1b8;","unicode":"","className":"fa-recycle"},
	{"unicodeEnt":"&#xf1a1;","unicode":"","className":"fa-reddit"},
	{"unicodeEnt":"&#xf281;","unicode":"","className":"fa-reddit-alien"},
	{"unicodeEnt":"&#xf1a2;","unicode":"","className":"fa-reddit-square"},
	{"unicodeEnt":"&#xf021;","unicode":"","className":"fa-refresh"},
	{"unicodeEnt":"&#xf25d;","unicode":"","className":"fa-registered"},
	{"unicodeEnt":"&#xf00d;","unicode":"","className":"fa-remove"},
	{"unicodeEnt":"&#xf18b;","unicode":"","className":"fa-renren"},
	{"unicodeEnt":"&#xf0c9;","unicode":"","className":"fa-reorder"},
	{"unicodeEnt":"&#xf01e;","unicode":"","className":"fa-repeat"},
	{"unicodeEnt":"&#xf112;","unicode":"","className":"fa-reply"},
	{"unicodeEnt":"&#xf122;","unicode":"","className":"fa-reply-all"},
	{"unicodeEnt":"&#xf079;","unicode":"","className":"fa-retweet"},
	{"unicodeEnt":"&#xf157;","unicode":"","className":"fa-rmb"},
	{"unicodeEnt":"&#xf018;","unicode":"","className":"fa-road"},
	{"unicodeEnt":"&#xf135;","unicode":"","className":"fa-rocket"},
	{"unicodeEnt":"&#xf0e2;","unicode":"","className":"fa-rotate-left"},
	{"unicodeEnt":"&#xf01e;","unicode":"","className":"fa-rotate-right"},
	{"unicodeEnt":"&#xf158;","unicode":"","className":"fa-rouble"},
	{"unicodeEnt":"&#xf09e;","unicode":"","className":"fa-rss"},
	{"unicodeEnt":"&#xf143;","unicode":"","className":"fa-rss-square"},
	{"unicodeEnt":"&#xf158;","unicode":"","className":"fa-rub"},
	{"unicodeEnt":"&#xf158;","unicode":"","className":"fa-ruble"},
	{"unicodeEnt":"&#xf156;","unicode":"","className":"fa-rupee"},
	{"unicodeEnt":"&#xf267;","unicode":"","className":"fa-safari"},
	{"unicodeEnt":"&#xf0c7;","unicode":"","className":"fa-save"},
	{"unicodeEnt":"&#xf0c4;","unicode":"","className":"fa-scissors"},
	{"unicodeEnt":"&#xf28a;","unicode":"","className":"fa-scribd"},
	{"unicodeEnt":"&#xf002;","unicode":"","className":"fa-search"},
	{"unicodeEnt":"&#xf010;","unicode":"","className":"fa-search-minus"},
	{"unicodeEnt":"&#xf00e;","unicode":"","className":"fa-search-plus"},
	{"unicodeEnt":"&#xf213;","unicode":"","className":"fa-sellsy"},
	{"unicodeEnt":"&#xf1d8;","unicode":"","className":"fa-send"},
	{"unicodeEnt":"&#xf1d9;","unicode":"","className":"fa-send-o"},
	{"unicodeEnt":"&#xf233;","unicode":"","className":"fa-server"},
	{"unicodeEnt":"&#xf064;","unicode":"","className":"fa-share"},
	{"unicodeEnt":"&#xf1e0;","unicode":"","className":"fa-share-alt"},
	{"unicodeEnt":"&#xf1e1;","unicode":"","className":"fa-share-alt-square"},
	{"unicodeEnt":"&#xf14d;","unicode":"","className":"fa-share-square"},
	{"unicodeEnt":"&#xf045;","unicode":"","className":"fa-share-square-o"},
	{"unicodeEnt":"&#xf20b;","unicode":"","className":"fa-shekel"},
	{"unicodeEnt":"&#xf20b;","unicode":"","className":"fa-sheqel"},
	{"unicodeEnt":"&#xf132;","unicode":"","className":"fa-shield"},
	{"unicodeEnt":"&#xf21a;","unicode":"","className":"fa-ship"},
	{"unicodeEnt":"&#xf214;","unicode":"","className":"fa-shirtsinbulk"},
	{"unicodeEnt":"&#xf290;","unicode":"","className":"fa-shopping-bag"},
	{"unicodeEnt":"&#xf291;","unicode":"","className":"fa-shopping-basket"},
	{"unicodeEnt":"&#xf07a;","unicode":"","className":"fa-shopping-cart"},
	{"unicodeEnt":"&#xf090;","unicode":"","className":"fa-sign-in"},
	{"unicodeEnt":"&#xf08b;","unicode":"","className":"fa-sign-out"},
	{"unicodeEnt":"&#xf012;","unicode":"","className":"fa-signal"},
	{"unicodeEnt":"&#xf215;","unicode":"","className":"fa-simplybuilt"},
	{"unicodeEnt":"&#xf0e8;","unicode":"","className":"fa-sitemap"},
	{"unicodeEnt":"&#xf216;","unicode":"","className":"fa-skyatlas"},
	{"unicodeEnt":"&#xf17e;","unicode":"","className":"fa-skype"},
	{"unicodeEnt":"&#xf198;","unicode":"","className":"fa-slack"},
	{"unicodeEnt":"&#xf1de;","unicode":"","className":"fa-sliders"},
	{"unicodeEnt":"&#xf1e7;","unicode":"","className":"fa-slideshare"},
	{"unicodeEnt":"&#xf118;","unicode":"","className":"fa-smile-o"},
	{"unicodeEnt":"&#xf1e3;","unicode":"","className":"fa-soccer-ball-o"},
	{"unicodeEnt":"&#xf0dc;","unicode":"","className":"fa-sort"},
	{"unicodeEnt":"&#xf15d;","unicode":"","className":"fa-sort-alpha-asc"},
	{"unicodeEnt":"&#xf15e;","unicode":"","className":"fa-sort-alpha-desc"},
	{"unicodeEnt":"&#xf160;","unicode":"","className":"fa-sort-amount-asc"},
	{"unicodeEnt":"&#xf161;","unicode":"","className":"fa-sort-amount-desc"},
	{"unicodeEnt":"&#xf0de;","unicode":"","className":"fa-sort-asc"},
	{"unicodeEnt":"&#xf0dd;","unicode":"","className":"fa-sort-desc"},
	{"unicodeEnt":"&#xf0dd;","unicode":"","className":"fa-sort-down"},
	{"unicodeEnt":"&#xf162;","unicode":"","className":"fa-sort-numeric-asc"},
	{"unicodeEnt":"&#xf163;","unicode":"","className":"fa-sort-numeric-desc"},
	{"unicodeEnt":"&#xf0de;","unicode":"","className":"fa-sort-up"},
	{"unicodeEnt":"&#xf1be;","unicode":"","className":"fa-soundcloud"},
	{"unicodeEnt":"&#xf197;","unicode":"","className":"fa-space-shuttle"},
	{"unicodeEnt":"&#xf110;","unicode":"","className":"fa-spinner"},
	{"unicodeEnt":"&#xf1b1;","unicode":"","className":"fa-spoon"},
	{"unicodeEnt":"&#xf1bc;","unicode":"","className":"fa-spotify"},
	{"unicodeEnt":"&#xf0c8;","unicode":"","className":"fa-square"},
	{"unicodeEnt":"&#xf096;","unicode":"","className":"fa-square-o"},
	{"unicodeEnt":"&#xf18d;","unicode":"","className":"fa-stack-exchange"},
	{"unicodeEnt":"&#xf16c;","unicode":"","className":"fa-stack-overflow"},
	{"unicodeEnt":"&#xf005;","unicode":"","className":"fa-star"},
	{"unicodeEnt":"&#xf089;","unicode":"","className":"fa-star-half"},
	{"unicodeEnt":"&#xf123;","unicode":"","className":"fa-star-half-empty"},
	{"unicodeEnt":"&#xf123;","unicode":"","className":"fa-star-half-full"},
	{"unicodeEnt":"&#xf123;","unicode":"","className":"fa-star-half-o"},
	{"unicodeEnt":"&#xf006;","unicode":"","className":"fa-star-o"},
	{"unicodeEnt":"&#xf1b6;","unicode":"","className":"fa-steam"},
	{"unicodeEnt":"&#xf1b7;","unicode":"","className":"fa-steam-square"},
	{"unicodeEnt":"&#xf048;","unicode":"","className":"fa-step-backward"},
	{"unicodeEnt":"&#xf051;","unicode":"","className":"fa-step-forward"},
	{"unicodeEnt":"&#xf0f1;","unicode":"","className":"fa-stethoscope"},
	{"unicodeEnt":"&#xf249;","unicode":"","className":"fa-sticky-note"},
	{"unicodeEnt":"&#xf24a;","unicode":"","className":"fa-sticky-note-o"},
	{"unicodeEnt":"&#xf04d;","unicode":"","className":"fa-stop"},
	{"unicodeEnt":"&#xf28d;","unicode":"","className":"fa-stop-circle"},
	{"unicodeEnt":"&#xf28e;","unicode":"","className":"fa-stop-circle-o"},
	{"unicodeEnt":"&#xf21d;","unicode":"","className":"fa-street-view"},
	{"unicodeEnt":"&#xf0cc;","unicode":"","className":"fa-strikethrough"},
	{"unicodeEnt":"&#xf1a4;","unicode":"","className":"fa-stumbleupon"},
	{"unicodeEnt":"&#xf1a3;","unicode":"","className":"fa-stumbleupon-circle"},
	{"unicodeEnt":"&#xf12c;","unicode":"","className":"fa-subscript"},
	{"unicodeEnt":"&#xf239;","unicode":"","className":"fa-subway"},
	{"unicodeEnt":"&#xf0f2;","unicode":"","className":"fa-suitcase"},
	{"unicodeEnt":"&#xf185;","unicode":"","className":"fa-sun-o"},
	{"unicodeEnt":"&#xf12b;","unicode":"","className":"fa-superscript"},
	{"unicodeEnt":"&#xf1cd;","unicode":"","className":"fa-support"},
	{"unicodeEnt":"&#xf0ce;","unicode":"","className":"fa-table"},
	{"unicodeEnt":"&#xf10a;","unicode":"","className":"fa-tablet"},
	{"unicodeEnt":"&#xf0e4;","unicode":"","className":"fa-tachometer"},
	{"unicodeEnt":"&#xf02b;","unicode":"","className":"fa-tag"},
	{"unicodeEnt":"&#xf02c;","unicode":"","className":"fa-tags"},
	{"unicodeEnt":"&#xf0ae;","unicode":"","className":"fa-tasks"},
	{"unicodeEnt":"&#xf1ba;","unicode":"","className":"fa-taxi"},
	{"unicodeEnt":"&#xf26c;","unicode":"","className":"fa-television"},
	{"unicodeEnt":"&#xf1d5;","unicode":"","className":"fa-tencent-weibo"},
	{"unicodeEnt":"&#xf120;","unicode":"","className":"fa-terminal"},
	{"unicodeEnt":"&#xf034;","unicode":"","className":"fa-text-height"},
	{"unicodeEnt":"&#xf035;","unicode":"","className":"fa-text-width"},
	{"unicodeEnt":"&#xf00a;","unicode":"","className":"fa-th"},
	{"unicodeEnt":"&#xf009;","unicode":"","className":"fa-th-large"},
	{"unicodeEnt":"&#xf00b;","unicode":"","className":"fa-th-list"},
	{"unicodeEnt":"&#xf08d;","unicode":"","className":"fa-thumb-tack"},
	{"unicodeEnt":"&#xf165;","unicode":"","className":"fa-thumbs-down"},
	{"unicodeEnt":"&#xf088;","unicode":"","className":"fa-thumbs-o-down"},
	{"unicodeEnt":"&#xf087;","unicode":"","className":"fa-thumbs-o-up"},
	{"unicodeEnt":"&#xf164;","unicode":"","className":"fa-thumbs-up"},
	{"unicodeEnt":"&#xf145;","unicode":"","className":"fa-ticket"},
	{"unicodeEnt":"&#xf00d;","unicode":"","className":"fa-times"},
	{"unicodeEnt":"&#xf057;","unicode":"","className":"fa-times-circle"},
	{"unicodeEnt":"&#xf05c;","unicode":"","className":"fa-times-circle-o"},
	{"unicodeEnt":"&#xf043;","unicode":"","className":"fa-tint"},
	{"unicodeEnt":"&#xf150;","unicode":"","className":"fa-toggle-down"},
	{"unicodeEnt":"&#xf191;","unicode":"","className":"fa-toggle-left"},
	{"unicodeEnt":"&#xf204;","unicode":"","className":"fa-toggle-off"},
	{"unicodeEnt":"&#xf205;","unicode":"","className":"fa-toggle-on"},
	{"unicodeEnt":"&#xf152;","unicode":"","className":"fa-toggle-right"},
	{"unicodeEnt":"&#xf151;","unicode":"","className":"fa-toggle-up"},
	{"unicodeEnt":"&#xf25c;","unicode":"","className":"fa-trademark"},
	{"unicodeEnt":"&#xf238;","unicode":"","className":"fa-train"},
	{"unicodeEnt":"&#xf224;","unicode":"","className":"fa-transgender"},
	{"unicodeEnt":"&#xf225;","unicode":"","className":"fa-transgender-alt"},
	{"unicodeEnt":"&#xf1f8;","unicode":"","className":"fa-trash"},
	{"unicodeEnt":"&#xf014;","unicode":"","className":"fa-trash-o"},
	{"unicodeEnt":"&#xf1bb;","unicode":"","className":"fa-tree"},
	{"unicodeEnt":"&#xf181;","unicode":"","className":"fa-trello"},
	{"unicodeEnt":"&#xf262;","unicode":"","className":"fa-tripadvisor"},
	{"unicodeEnt":"&#xf091;","unicode":"","className":"fa-trophy"},
	{"unicodeEnt":"&#xf0d1;","unicode":"","className":"fa-truck"},
	{"unicodeEnt":"&#xf195;","unicode":"","className":"fa-try"},
	{"unicodeEnt":"&#xf1e4;","unicode":"","className":"fa-tty"},
	{"unicodeEnt":"&#xf173;","unicode":"","className":"fa-tumblr"},
	{"unicodeEnt":"&#xf174;","unicode":"","className":"fa-tumblr-square"},
	{"unicodeEnt":"&#xf195;","unicode":"","className":"fa-turkish-lira"},
	{"unicodeEnt":"&#xf26c;","unicode":"","className":"fa-tv"},
	{"unicodeEnt":"&#xf1e8;","unicode":"","className":"fa-twitch"},
	{"unicodeEnt":"&#xf099;","unicode":"","className":"fa-twitter"},
	{"unicodeEnt":"&#xf081;","unicode":"","className":"fa-twitter-square"},
	{"unicodeEnt":"&#xf0e9;","unicode":"","className":"fa-umbrella"},
	{"unicodeEnt":"&#xf0cd;","unicode":"","className":"fa-underline"},
	{"unicodeEnt":"&#xf0e2;","unicode":"","className":"fa-undo"},
	{"unicodeEnt":"&#xf19c;","unicode":"","className":"fa-university"},
	{"unicodeEnt":"&#xf127;","unicode":"","className":"fa-unlink"},
	{"unicodeEnt":"&#xf09c;","unicode":"","className":"fa-unlock"},
	{"unicodeEnt":"&#xf13e;","unicode":"","className":"fa-unlock-alt"},
	{"unicodeEnt":"&#xf0dc;","unicode":"","className":"fa-unsorted"},
	{"unicodeEnt":"&#xf093;","unicode":"","className":"fa-upload"},
	{"unicodeEnt":"&#xf287;","unicode":"","className":"fa-usb"},
	{"unicodeEnt":"&#xf155;","unicode":"","className":"fa-usd"},
	{"unicodeEnt":"&#xf007;","unicode":"","className":"fa-user"},
	{"unicodeEnt":"&#xf0f0;","unicode":"","className":"fa-user-md"},
	{"unicodeEnt":"&#xf234;","unicode":"","className":"fa-user-plus"},
	{"unicodeEnt":"&#xf21b;","unicode":"","className":"fa-user-secret"},
	{"unicodeEnt":"&#xf235;","unicode":"","className":"fa-user-times"},
	{"unicodeEnt":"&#xf0c0;","unicode":"","className":"fa-users"},
	{"unicodeEnt":"&#xf221;","unicode":"","className":"fa-venus"},
	{"unicodeEnt":"&#xf226;","unicode":"","className":"fa-venus-double"},
	{"unicodeEnt":"&#xf228;","unicode":"","className":"fa-venus-mars"},
	{"unicodeEnt":"&#xf237;","unicode":"","className":"fa-viacoin"},
	{"unicodeEnt":"&#xf03d;","unicode":"","className":"fa-video-camera"},
	{"unicodeEnt":"&#xf27d;","unicode":"","className":"fa-vimeo"},
	{"unicodeEnt":"&#xf194;","unicode":"","className":"fa-vimeo-square"},
	{"unicodeEnt":"&#xf1ca;","unicode":"","className":"fa-vine"},
	{"unicodeEnt":"&#xf189;","unicode":"","className":"fa-vk"},
	{"unicodeEnt":"&#xf027;","unicode":"","className":"fa-volume-down"},
	{"unicodeEnt":"&#xf026;","unicode":"","className":"fa-volume-off"},
	{"unicodeEnt":"&#xf028;","unicode":"","className":"fa-volume-up"},
	{"unicodeEnt":"&#xf071;","unicode":"","className":"fa-warning"},
	{"unicodeEnt":"&#xf1d7;","unicode":"","className":"fa-wechat"},
	{"unicodeEnt":"&#xf18a;","unicode":"","className":"fa-weibo"},
	{"unicodeEnt":"&#xf1d7;","unicode":"","className":"fa-weixin"},
	{"unicodeEnt":"&#xf232;","unicode":"","className":"fa-whatsapp"},
	{"unicodeEnt":"&#xf193;","unicode":"","className":"fa-wheelchair"},
	{"unicodeEnt":"&#xf1eb;","unicode":"","className":"fa-wifi"},
	{"unicodeEnt":"&#xf266;","unicode":"","className":"fa-wikipedia-w"},
	{"unicodeEnt":"&#xf17a;","unicode":"","className":"fa-windows"},
	{"unicodeEnt":"&#xf159;","unicode":"","className":"fa-won"},
	{"unicodeEnt":"&#xf19a;","unicode":"","className":"fa-wordpress"},
	{"unicodeEnt":"&#xf0ad;","unicode":"","className":"fa-wrench"},
	{"unicodeEnt":"&#xf168;","unicode":"","className":"fa-xing"},
	{"unicodeEnt":"&#xf169;","unicode":"","className":"fa-xing-square"},
	{"unicodeEnt":"&#xf23b;","unicode":"","className":"fa-y-combinator"},
	{"unicodeEnt":"&#xf1d4;","unicode":"","className":"fa-y-combinator-square"},
	{"unicodeEnt":"&#xf19e;","unicode":"","className":"fa-yahoo"},
	{"unicodeEnt":"&#xf23b;","unicode":"","className":"fa-yc"},
	{"unicodeEnt":"&#xf1d4;","unicode":"","className":"fa-yc-square"},
	{"unicodeEnt":"&#xf1e9;","unicode":"","className":"fa-yelp"},
	{"unicodeEnt":"&#xf157;","unicode":"","className":"fa-yen"},
	{"unicodeEnt":"&#xf167;","unicode":"","className":"fa-youtube"},
	{"unicodeEnt":"&#xf16a;","unicode":"","className":"fa-youtube-play"},
	{"unicodeEnt":"&#xf166;","unicode":"","className":"fa-youtube-square"}
	]
/**
 *
 * @param hex The hex color value with opacity.
 * @returns {string|*}
 */
function convertHexToRGBA(hex){
	hex = hex.replace('0X','');
	var opacity = parseInt(hex.substring(0,2), 16);
	var r = parseInt(hex.substring(2,4), 16);
	var g = parseInt(hex.substring(4,6), 16);
	var b = parseInt(hex.substring(6,8), 16);

	return 'rgba('+ r +','+ g +','+ b +','+ opacity/255 +')';
}

/**
 *
 * @param hex Returns a DreemGL vec4 for this color.
 * @returns {string|*}
 */
function convertToVec4(hex){
	hex = hex.replace('0X','');
	var opacity = parseInt(hex.substring(0,2), 16) / 255;
	var r = Number(parseInt(hex.substring(2,4), 16) / 255).toFixed(3);
	var g = Number(parseInt(hex.substring(4,6), 16) / 255).toFixed(3);
	var b = Number(parseInt(hex.substring(6,8), 16) / 255).toFixed(3);

	return 'vec4('+ r +','+ g +','+ b +','+ opacity +')';
}

function createColorTable() {
	var content = "<table>" +
		"<tr><th>Color name</th><th>Color</th><th>HEX</th><th>RGBA CSS value</th><th>vec4</th></tr>\n";

	for (var k in color_wikipedia) {
		var hexColor = color_wikipedia[ k].toUpperCase();
		var rgbaVal = convertHexToRGBA(hexColor);
		var vec4Val = convertToVec4(hexColor);
		var line = "\t<tr>\n"
			+ "\t\t<td>" + k + "</td>\n"
			+ "\t\t<td style='background-color: " + rgbaVal + "'></td>\n"
			+ "\t\t<td style='font-family: \"Lucida Console\", Courier, \"Courier New\"'>" + hexColor.replace("0X", "") + "</td>\n"
			+ "\t\t<td style='font-family: \"Lucida Console\", Courier, \"Courier New\"'>" + rgbaVal + "\t\t</td>\n"
			+ "\t\t<td style='font-family: \"Lucida Console\", Courier, \"Courier New\"'>" + vec4Val + "\t\t</td>\n"
			+ "\t</tr>\n";
		content += line;
	}
	content += "</table>\n\n"
	return content;
}

function createFontAwesomeTable() {
	var content = '### Font Awesome \n\nFont Awesome 4.5.0 Icon, CSS Class, & Unicode:\n\n';
	content += "<table>\n" +
		  "<tr><th colspan='4'>Font Awesome - The iconic font and CSS toolkit</th></tr>\n";

	// 2 items per row
	for (var i=0; i< Math.ceil(fontAwesome.length / 3); i++) {
		var line = "\t<tr>\n";
		var icon = null;
		for (var j=0; j<3; j++) {
			// structure: {"unicodeEnt":"&#xf166;","unicode":"","className":"fa-youtube-square"}
			if (fontAwesome.length > i*3 + j) {
				icon = fontAwesome[ i*3 + j ]
				line += '\t\t<td><span  style="font-family: FontAwesome">' + icon.unicodeEnt + '</span>&nbsp;&nbsp;' + icon.className + '&nbsp;&nbsp;[' + icon.unicodeEnt.replace('&', '&amp;') + ']</td>\n';
			} else {
				line += '\t\t<td>&nbsp;</td>'
			}
		}
		line += "\t</tr>\n"
		content += line;
	}
	content += "</table>\n\n"
	console.log(content);
	return content;
}

var markdown = fs.readFileSync('README_template.md');
markdown += createColorTable();
markdown += createFontAwesomeTable();

fs.writeFile('README.md', markdown, (err) => {
	if (err) {
		throw err;
	}
	console.log('New version of README.md for appendix written!');
});
